# Copyright 2012 OpenStack Foundation
# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import collections
import email.utils
import re
import time
import urllib

import jsonschema
from oslo_log import log as logging
from oslo_log import versionutils
from oslo_serialization import jsonutils as json
import six

from tempest.lib.common import http
from tempest.lib.common import jsonschema_validator
from tempest.lib.common import profiler
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions

# redrive rate limited calls at most twice
MAX_RECURSION_DEPTH = 2

# All the successful HTTP status codes from RFC 7231 & 4918
HTTP_SUCCESS = (200, 201, 202, 203, 204, 205, 206, 207)

# All the redirection HTTP status codes from RFC 7231 & 4918
HTTP_REDIRECTION = (300, 301, 302, 303, 304, 305, 306, 307)

# JSON Schema validator and format checker used for JSON Schema validation
JSONSCHEMA_VALIDATOR = jsonschema_validator.JSONSCHEMA_VALIDATOR
FORMAT_CHECKER = jsonschema_validator.FORMAT_CHECKER


class RestClient(object):
    """Unified OpenStack RestClient class

    This class is used for building openstack api clients on top of. It is
    intended to provide a base layer for wrapping outgoing http requests in
    keystone auth as well as providing response code checking and error
    handling.

    :param auth_provider: an auth provider object used to wrap requests in auth
    :param str service: The service name to use for the catalog lookup
    :param str region: The region to use for the catalog lookup
    :param str name: The endpoint name to use for the catalog lookup; this
                     returns only if the service exists
    :param str endpoint_type: The endpoint type to use for the catalog lookup
    :param int build_interval: Time in seconds between to status checks in
                               wait loops
    :param int build_timeout: Timeout in seconds to wait for a wait operation.
    :param bool disable_ssl_certificate_validation: Set to true to disable ssl
                                                    certificate validation
    :param str ca_certs: File containing the CA Bundle to use in verifying a
                         TLS server cert
    :param str trace_requests: Regex to use for specifying logging the entirety
                              of the request and response payload
    :param str http_timeout: Timeout in seconds to wait for the http request to
                             return
    :param str proxy_url: http proxy url to use.
    :param bool follow_redirects: Set to false to stop following redirects.
    """

    # The version of the API this client implements
    api_version = None

    LOG = logging.getLogger(__name__)

    def __init__(self, auth_provider, service, region,
                 endpoint_type='publicURL',
                 build_interval=1, build_timeout=60,
                 disable_ssl_certificate_validation=False, ca_certs=None,
                 trace_requests='', name=None, http_timeout=None,
                 proxy_url=None, follow_redirects=True):
        self.auth_provider = auth_provider
        self.service = service
        self.region = region
        self.name = name
        self.endpoint_type = endpoint_type
        self.build_interval = build_interval
        self.build_timeout = build_timeout
        self.trace_requests = trace_requests

        self._skip_path = False
        self.general_header_lc = set(('cache-control', 'connection',
                                      'date', 'pragma', 'trailer',
                                      'transfer-encoding', 'via',
                                      'warning'))
        self.response_header_lc = set(('accept-ranges', 'age', 'etag',
                                       'location', 'proxy-authenticate',
                                       'retry-after', 'server',
                                       'vary', 'www-authenticate'))
        self.dscv = disable_ssl_certificate_validation

        if proxy_url:
            self.http_obj = http.ClosingProxyHttp(
                proxy_url,
                disable_ssl_certificate_validation=self.dscv,
                ca_certs=ca_certs,
                timeout=http_timeout, follow_redirects=follow_redirects)
        else:
            self.http_obj = http.ClosingHttp(
                disable_ssl_certificate_validation=self.dscv,
                ca_certs=ca_certs,
                timeout=http_timeout, follow_redirects=follow_redirects)

    def get_headers(self, accept_type=None, send_type=None):
        """Return the default headers which will be used with outgoing requests

        :param str accept_type: The media type to use for the Accept header, if
                                one isn't provided the object var TYPE will be
                                used
        :param str send_type: The media-type to use for the Content-Type
                              header, if one isn't provided the object var
                              TYPE will be used
        :rtype: dict
        :return: The dictionary of headers which can be used in the headers
                 dict for outgoing request
        """
        if accept_type is None:
            accept_type = 'json'
        if send_type is None:
            send_type = 'json'
        headers = {'Content-Type': 'application/%s' % send_type,
                   'Accept': 'application/%s' % accept_type}
        headers.update(profiler.serialize_as_http_headers())
        return headers

    def __str__(self):
        STRING_LIMIT = 80
        str_format = ("service:%s, base_url:%s, "
                      "filters: %s, build_interval:%s, build_timeout:%s"
                      "\ntoken:%s..., \nheaders:%s...")
        return str_format % (self.service, self.base_url,
                             self.filters, self.build_interval,
                             self.build_timeout,
                             str(self.token)[0:STRING_LIMIT],
                             str(self.get_headers())[0:STRING_LIMIT])

    @property
    def user(self):
        """The username used for requests

        :rtype: string
        :return: The username being used for requests
        """

        return self.auth_provider.credentials.username

    @property
    def user_id(self):
        """The user_id used for requests

        :rtype: string
        :return: The user id being used for requests
        """
        return self.auth_provider.credentials.user_id

    @property
    def tenant_name(self):
        """The tenant/project being used for requests

        :rtype: string
        :return: The tenant/project name being used for requests
        """
        return self.auth_provider.credentials.tenant_name

    @property
    def project_id(self):
        """The project id being used for requests

        :rtype: string
        :return: The project id being used for requests
        """
        return self.auth_provider.credentials.tenant_id

    @property
    def tenant_id(self):
        """The tenant/project id being used for requests

        :rtype: string
        :return: The tenant/project id being used for requests
        """
        # NOTE(ralonsoh): this property should be deprecated, reference
        # blueprint adopt-oslo-versioned-objects-for-db.
        versionutils.report_deprecated_feature(
            self.LOG, '"tenant_id" property is deprecated for removal, use '
                      '"project_id" instead')
        return self.project_id

    @property
    def password(self):
        """The password being used for requests

        :rtype: string
        :return: The password being used for requests
        """
        return self.auth_provider.credentials.password

    @property
    def base_url(self):
        return self.auth_provider.base_url(filters=self.filters)

    @property
    def token(self):
        return self.auth_provider.get_token()

    @property
    def filters(self):
        _filters = dict(
            service=self.service,
            endpoint_type=self.endpoint_type,
            region=self.region,
            name=self.name
        )
        if self.api_version is not None:
            _filters['api_version'] = self.api_version
        if self._skip_path:
            _filters['skip_path'] = self._skip_path
        return _filters

    def skip_path(self):
        """When set, ignore the path part of the base URL from the catalog"""
        self._skip_path = True

    def reset_path(self):
        """When reset, use the base URL from the catalog as-is"""
        self._skip_path = False

    @classmethod
    def expected_success(cls, expected_code, read_code):
        """Check expected success response code against the http response

        :param int expected_code: The response code that is expected.
                                  Optionally a list of integers can be used
                                  to specify multiple valid success codes
        :param int read_code: The response code which was returned in the
                              response
        :raises AssertionError: if the expected_code isn't a valid http success
                                response code
        :raises exceptions.InvalidHttpSuccessCode: if the read code isn't an
                                                   expected http success code
        """
        if not isinstance(read_code, int):
            raise TypeError("'read_code' must be an int instead of (%s)"
                            % type(read_code))

        assert_msg = ("This function only allowed to use for HTTP status "
                      "codes which explicitly defined in the RFC 7231 & 4918. "
                      "{0} is not a defined Success Code!"
                      ).format(expected_code)
        if isinstance(expected_code, list):
            for code in expected_code:
                assert code in HTTP_SUCCESS + HTTP_REDIRECTION, assert_msg
        else:
            assert expected_code in HTTP_SUCCESS + HTTP_REDIRECTION, assert_msg

        # NOTE(afazekas): the http status code above 400 is processed by
        # the _error_checker method
        if read_code < 400:
            pattern = ("Unexpected http success status code {0}, "
                       "The expected status code is {1}")
            if ((not isinstance(expected_code, list) and
                 (read_code != expected_code)) or
                (isinstance(expected_code, list) and
                 (read_code not in expected_code))):
                details = pattern.format(read_code, expected_code)
                raise exceptions.InvalidHttpSuccessCode(details)

    def post(self, url, body, headers=None, extra_headers=False,
             chunked=False):
        """Send a HTTP POST request using keystone auth

        :param str url: the relative url to send the post request to
        :param dict body: the request body
        :param dict headers: The headers to use for the request
        :param bool extra_headers: Boolean value than indicates if the headers
                                   returned by the get_headers() method are to
                                   be used but additional headers are needed in
                                   the request pass them in as a dict.
        :param bool chunked: sends the body with chunked encoding
        :return: a tuple with the first entry containing the response headers
                 and the second the response body
        :rtype: tuple
        """
        return self.request('POST', url, extra_headers, headers, body, chunked)

    def get(self, url, headers=None, extra_headers=False):
        """Send a HTTP GET request using keystone service catalog and auth

        :param str url: the relative url to send the get request to
        :param dict headers: The headers to use for the request
        :param bool extra_headers: Boolean value than indicates if the headers
                                   returned by the get_headers() method are to
                                   be used but additional headers are needed in
                                   the request pass them in as a dict.
        :return: a tuple with the first entry containing the response headers
                 and the second the response body
        :rtype: tuple
        """
        return self.request('GET', url, extra_headers, headers)

    def delete(self, url, headers=None, body=None, extra_headers=False):
        """Send a HTTP DELETE request using keystone service catalog and auth

        :param str url: the relative url to send the delete request to
        :param dict headers: The headers to use for the request
        :param dict body: the request body
        :param bool extra_headers: Boolean value than indicates if the headers
                                   returned by the get_headers() method are to
                                   be used but additional headers are needed in
                                   the request pass them in as a dict.
        :return: a tuple with the first entry containing the response headers
                 and the second the response body
        :rtype: tuple
        """
        return self.request('DELETE', url, extra_headers, headers, body)

    def patch(self, url, body, headers=None, extra_headers=False):
        """Send a HTTP PATCH request using keystone service catalog and auth

        :param str url: the relative url to send the patch request to
        :param dict body: the request body
        :param dict headers: The headers to use for the request
        :param bool extra_headers: Boolean value than indicates if the headers
                                   returned by the get_headers() method are to
                                   be used but additional headers are needed in
                                   the request pass them in as a dict.
        :return: a tuple with the first entry containing the response headers
                 and the second the response body
        :rtype: tuple
        """
        return self.request('PATCH', url, extra_headers, headers, body)

    def put(self, url, body, headers=None, extra_headers=False, chunked=False):
        """Send a HTTP PUT request using keystone service catalog and auth

        :param str url: the relative url to send the put request to
        :param dict body: the request body
        :param dict headers: The headers to use for the request
        :param bool extra_headers: Boolean value than indicates if the headers
                                   returned by the get_headers() method are to
                                   be used but additional headers are needed in
                                   the request pass them in as a dict.
        :param bool chunked: sends the body with chunked encoding
        :return: a tuple with the first entry containing the response headers
                 and the second the response body
        :rtype: tuple
        """
        return self.request('PUT', url, extra_headers, headers, body, chunked)

    def head(self, url, headers=None, extra_headers=False):
        """Send a HTTP HEAD request using keystone service catalog and auth

        :param str url: the relative url to send the head request to
        :param dict headers: The headers to use for the request
        :param bool extra_headers: Boolean value than indicates if the headers
                                   returned by the get_headers() method are to
                                   be used but additional headers are needed in
                                   the request pass them in as a dict.
        :return: a tuple with the first entry containing the response headers
                 and the second the response body
        :rtype: tuple
        """
        return self.request('HEAD', url, extra_headers, headers)

    def copy(self, url, headers=None, extra_headers=False):
        """Send a HTTP COPY request using keystone service catalog and auth

        :param str url: the relative url to send the copy request to
        :param dict headers: The headers to use for the request
        :param bool extra_headers: Boolean value than indicates if the headers
                                   returned by the get_headers() method are to
                                   be used but additional headers are needed in
                                   the request pass them in as a dict.
        :return: a tuple with the first entry containing the response headers
                 and the second the response body
        :rtype: tuple
        """
        return self.request('COPY', url, extra_headers, headers)

    def get_versions(self):
        """Get the versions on an endpoint from the keystone catalog

        This method will make a GET request on the baseurl from the keystone
        catalog to return a list of API versions. It is expected that a GET
        on the endpoint in the catalog will return a list of supported API
        versions.

        :return: tuple with response headers and list of version numbers
        :rtype: tuple
        """
        resp, body = self.get('')
        body = self._parse_resp(body)
        versions = map(lambda x: x['id'], body)
        return resp, versions

    def _get_request_id(self, resp):
        for i in ('x-openstack-request-id', 'x-compute-request-id'):
            if i in resp:
                return resp[i]
        return ""

    def _safe_body(self, body, maxlen=4096):
        # convert a structure into a string safely
        try:
            text = str(body)
        except UnicodeDecodeError:
            # if this isn't actually text, return marker that
            return "<BinaryData: removed>"
        if len(text) > maxlen:
            return text[:maxlen]
        else:
            return text

    def _log_request_start(self, method, req_url):
        caller_name = test_utils.find_test_caller()
        if self.trace_requests and re.search(self.trace_requests, caller_name):
            self.LOG.debug('Starting Request (%s): %s %s', caller_name,
                           method, req_url)

    def _log_request_full(self, resp, req_headers=None, req_body=None,
                          resp_body=None, extra=None):
        if 'X-Auth-Token' in req_headers:
            req_headers['X-Auth-Token'] = '<omitted>'
        if 'X-Subject-Token' in req_headers:
            req_headers['X-Subject-Token'] = '<omitted>'
        # A shallow copy is sufficient
        resp_log = resp.copy()
        if 'x-subject-token' in resp_log:
            resp_log['x-subject-token'] = '<omitted>'
        log_fmt = """Request - Headers: %s
        Body: %s
    Response - Headers: %s
        Body: %s"""

        self.LOG.debug(
            log_fmt,
            str(req_headers),
            self._safe_body(req_body),
            str(resp_log),
            self._safe_body(resp_body),
            extra=extra)

    def _log_request(self, method, req_url, resp,
                     secs="", req_headers=None,
                     req_body=None, resp_body=None):
        if req_headers is None:
            req_headers = {}
        # if we have the request id, put it in the right part of the log
        extra = dict(request_id=self._get_request_id(resp))
        # NOTE(sdague): while we still have 6 callers to this function
        # we're going to just provide work around on who is actually
        # providing timings by gracefully adding no content if they don't.
        # Once we're down to 1 caller, clean this up.
        caller_name = test_utils.find_test_caller()
        if secs:
            secs = " %.3fs" % secs
        self.LOG.info(
            'Request (%s): %s %s %s%s',
            caller_name,
            resp['status'],
            method,
            req_url,
            secs,
            extra=extra)

        # Also look everything at DEBUG if you want to filter this
        # out, don't run at debug.
        if self.LOG.isEnabledFor(logging.DEBUG):
            self._log_request_full(resp, req_headers, req_body,
                                   resp_body, extra)

    def _parse_resp(self, body):
        try:
            body = json.loads(body)
        except ValueError:
            return body

        # We assume, that if the first value of the deserialized body's
        # item set is a dict or a list, that we just return the first value
        # of deserialized body.
        # Essentially "cutting out" the first placeholder element in a body
        # that looks like this:
        #
        #  {
        #    "users": [
        #      ...
        #    ]
        #  }
        try:
            # Ensure there are not more than one top-level keys
            # NOTE(freerunner): Ensure, that JSON is not nullable to
            # to prevent StopIteration Exception
            if not hasattr(body, "keys") or len(body.keys()) != 1:
                return body
            # Just return the "wrapped" element
            _, first_item = six.next(six.iteritems(body))
            if isinstance(first_item, (dict, list)):
                return first_item
        except (ValueError, IndexError):
            pass
        return body

    def response_checker(self, method, resp, resp_body):
        """A sanity check on the response from a HTTP request

        This method does a sanity check on whether the response from an HTTP
        request conforms the HTTP RFC.

        :param str method: The HTTP verb of the request associated with the
                           response being passed in.
        :param resp: The response headers
        :param resp_body: The body of the response
        :raises ResponseWithNonEmptyBody: If the response with the status code
                                          is not supposed to have a body
        :raises ResponseWithEntity: If the response code is 205 but has an
                                    entity
        """
        if (resp.status in set((204, 205, 304)) or resp.status < 200 or
                method.upper() == 'HEAD') and resp_body:
            raise exceptions.ResponseWithNonEmptyBody(status=resp.status)
        # NOTE(afazekas):
        # If the HTTP Status Code is 205
        #   'The response MUST NOT include an entity.'
        # A HTTP entity has an entity-body and an 'entity-header'.
        # In the HTTP response specification (Section 6) the 'entity-header'
        # 'generic-header' and 'response-header' are in OR relation.
        # All headers not in the above two group are considered as entity
        # header in every interpretation.

        if (resp.status == 205 and
            0 != len(set(resp.keys()) - set(('status',)) -
                     self.response_header_lc - self.general_header_lc)):
            raise exceptions.ResponseWithEntity()
        # NOTE(afazekas)
        # Now the swift sometimes (delete not empty container)
        # returns with non json error response, we can create new rest class
        # for swift.
        # Usually RFC2616 says error responses SHOULD contain an explanation.
        # The warning is normal for SHOULD/SHOULD NOT case

        # Likely it will cause an error
        if method != 'HEAD' and not resp_body and resp.status >= 400:
            self.LOG.warning("status >= 400 response with empty body")

    def _request(self, method, url, headers=None, body=None, chunked=False):
        """A simple HTTP request interface."""
        # Authenticate the request with the auth provider
        req_url, req_headers, req_body = self.auth_provider.auth_request(
            method, url, headers, body, self.filters)

        resp, resp_body = self.raw_request(
            req_url, method, headers=req_headers, body=req_body,
            chunked=chunked
        )
        # Verify HTTP response codes
        self.response_checker(method, resp, resp_body)

        return resp, resp_body

    def raw_request(self, url, method, headers=None, body=None, chunked=False,
                    log_req_body=None):
        """Send a raw HTTP request without the keystone catalog or auth

        This method sends a HTTP request in the same manner as the request()
        method, however it does so without using keystone auth or the catalog
        to determine the base url. Additionally no response handling is done
        the results from the request are just returned.

        :param str url: Full url to send the request
        :param str method: The HTTP verb to use for the request
        :param dict headers: Headers to use for the request. If none are
                             specified, then the headers returned from the
                             get_headers() method are used. If the request
                             explicitly requires no headers use an empty dict.
        :param str body: Body to send with the request
        :param bool chunked: sends the body with chunked encoding
        :param str log_req_body: Whether to log the request body or not.
                                 It is default to None which means request
                                 body is safe to log otherwise pass any string
                                 you want to log in place of request body.
                                 For example: '<omitted>'
        :rtype: tuple
        :return: a tuple with the first entry containing the response headers
                 and the second the response body
        """
        if headers is None:
            headers = self.get_headers()
        # Do the actual request, and time it
        start = time.time()
        self._log_request_start(method, url)
        resp, resp_body = self.http_obj.request(
            url, method, headers=headers,
            body=body, chunked=chunked)
        end = time.time()
        req_body = body if log_req_body is None else log_req_body
        self._log_request(method, url, resp, secs=(end - start),
                          req_headers=headers, req_body=req_body,
                          resp_body=resp_body)
        return resp, resp_body

    def request(self, method, url, extra_headers=False, headers=None,
                body=None, chunked=False):
        """Send a HTTP request with keystone auth and using the catalog

        This method will send an HTTP request using keystone auth in the
        headers and the catalog to determine the endpoint to use for the
        baseurl to send the request to. Additionally

        When a response is received it will check it to see if an error
        response was received. If it was an exception will be raised to enable
        it to be handled quickly.

        This method will also handle rate-limiting, if a 413 response code is
        received it will retry the request after waiting the 'retry-after'
        duration from the header.

        :param str method: The HTTP verb to use for the request
        :param str url: Relative url to send the request to
        :param bool extra_headers: Boolean value than indicates if the headers
                                   returned by the get_headers() method are to
                                   be used but additional headers are needed in
                                   the request pass them in as a dict.
        :param dict headers: Headers to use for the request. If none are
                             specified, then the headers returned from the
                             get_headers() method are used. If the request
                             explicitly requires no headers use an empty dict.
        :param str body: Body to send with the request
        :param bool chunked: sends the body with chunked encoding
        :rtype: tuple
        :return: a tuple with the first entry containing the response headers
                 and the second the response body
        :raises UnexpectedContentType: If the content-type of the response
                                       isn't an expect type
        :raises Unauthorized: If a 401 response code is received
        :raises Forbidden: If a 403 response code is received
        :raises NotFound: If a 404 response code is received
        :raises BadRequest: If a 400 response code is received
        :raises Gone: If a 410 response code is received
        :raises Conflict: If a 409 response code is received
        :raises PreconditionFailed: If a 412 response code is received
        :raises OverLimit: If a 413 response code is received and retry-after
                           is not in the response body or its retry operation
                           exceeds the limits defined by the server
        :raises RateLimitExceeded: If a 413 response code is received and
                                   retry-after is in the response body and
                                   its retry operation does not exceeds the
                                   limits defined by the server
        :raises InvalidContentType: If a 415 response code is received
        :raises UnprocessableEntity: If a 422 response code is received
        :raises InvalidHTTPResponseBody: The response body wasn't valid JSON
                                         and couldn't be parsed
        :raises NotImplemented: If a 501 response code is received
        :raises ServerFault: If a 500 response code is received
        :raises UnexpectedResponseCode: If a response code above 400 is
                                        received and it doesn't fall into any
                                        of the handled checks
        """
        # if extra_headers is True
        # default headers would be added to headers
        retry = 0

        if headers is None:
            # NOTE(vponomaryov): if some client do not need headers,
            # it should explicitly pass empty dict
            headers = self.get_headers()
        elif extra_headers:
            try:
                headers.update(self.get_headers())
            except (ValueError, TypeError):
                headers = self.get_headers()

        resp, resp_body = self._request(method, url, headers=headers,
                                        body=body, chunked=chunked)

        while (resp.status == 413 and
               'retry-after' in resp and
                not self.is_absolute_limit(
                    resp, self._parse_resp(resp_body)) and
                retry < MAX_RECURSION_DEPTH):
            retry += 1
            delay = self._get_retry_after_delay(resp)
            self.LOG.debug(
                "Sleeping %s seconds based on retry-after header", delay
            )
            time.sleep(delay)
            resp, resp_body = self._request(method, url,
                                            headers=headers, body=body)
        self._error_checker(resp, resp_body)
        return resp, resp_body

    def _get_retry_after_delay(self, resp):
        """Extract the delay from the retry-after header.

        This supports both integer and HTTP date formatted retry-after headers
        per RFC 2616.

        :param resp: The response containing the retry-after headers
        :rtype: int
        :return: The delay in seconds, clamped to be at least 1 second
        :raises ValueError: On failing to parse the delay
        """
        delay = None
        try:
            delay = int(resp['retry-after'])
        except (ValueError, KeyError):
            pass

        try:
            retry_timestamp = self._parse_http_date(resp['retry-after'])
            date_timestamp = self._parse_http_date(resp['date'])
            delay = int(retry_timestamp - date_timestamp)
        except (ValueError, OverflowError, KeyError):
            pass

        if delay is None:
            raise ValueError(
                "Failed to parse retry-after header %r as either int or "
                "HTTP-date." % resp.get('retry-after')
            )

        # Retry-after headers do not have sub-second precision. Clients may
        # receive a delay of 0. After sleeping 0 seconds, we would (likely) hit
        # another 413. To avoid this, always sleep at least 1 second.
        return max(1, delay)

    def _parse_http_date(self, val):
        """Parse an HTTP date, like 'Fri, 31 Dec 1999 23:59:59 GMT'.

        Return an epoch timestamp (float), as returned by time.mktime().
        """
        parts = email.utils.parsedate(val)
        if not parts:
            raise ValueError("Failed to parse date %s" % val)
        return time.mktime(parts)

    def _error_checker(self, resp, resp_body):

        # NOTE(mtreinish): Check for httplib response from glance_http. The
        # object can't be used here because importing httplib breaks httplib2.
        # If another object from a class not imported were passed here as
        # resp this could possibly fail
        if str(type(resp)) == "<type 'instance'>":
            ctype = resp.getheader('content-type')
        else:
            try:
                ctype = resp['content-type']
            # NOTE(mtreinish): Keystone delete user responses doesn't have a
            # content-type header. (They don't have a body) So just pretend it
            # is set.
            except KeyError:
                ctype = 'application/json'

        # It is not an error response
        if resp.status < 400:
            return

        # NOTE(zhipengh): There is a purposefully duplicate of content-type
        # with the only difference is with or without spaces, as specified
        # in RFC7231.
        JSON_ENC = ['application/json', 'application/json; charset=utf-8',
                    'application/json;charset=utf-8']

        # NOTE(mtreinish): This is for compatibility with Glance and swift
        # APIs. These are the return content types that Glance api v1
        # (and occasionally swift) are using.
        # NOTE(zhipengh): There is a purposefully duplicate of content-type
        # with the only difference is with or without spaces, as specified
        # in RFC7231.
        TXT_ENC = ['text/plain', 'text/html', 'text/html; charset=utf-8',
                   'text/plain; charset=utf-8', 'text/html;charset=utf-8',
                   'text/plain;charset=utf-8']

        if ctype.lower() in JSON_ENC:
            parse_resp = True
        elif ctype.lower() in TXT_ENC:
            parse_resp = False
        else:
            raise exceptions.UnexpectedContentType(str(resp.status),
                                                   resp=resp)

        if resp.status == 401:
            if parse_resp:
                resp_body = self._parse_resp(resp_body)
            raise exceptions.Unauthorized(resp_body, resp=resp)

        if resp.status == 403:
            if parse_resp:
                resp_body = self._parse_resp(resp_body)
            raise exceptions.Forbidden(resp_body, resp=resp)

        if resp.status == 404:
            if parse_resp:
                resp_body = self._parse_resp(resp_body)
            raise exceptions.NotFound(resp_body, resp=resp)

        if resp.status == 400:
            if parse_resp:
                resp_body = self._parse_resp(resp_body)
            raise exceptions.BadRequest(resp_body, resp=resp)

        if resp.status == 410:
            if parse_resp:
                resp_body = self._parse_resp(resp_body)
            raise exceptions.Gone(resp_body, resp=resp)

        if resp.status == 409:
            if parse_resp:
                resp_body = self._parse_resp(resp_body)
            raise exceptions.Conflict(resp_body, resp=resp)

        if resp.status == 412:
            if parse_resp:
                resp_body = self._parse_resp(resp_body)
            raise exceptions.PreconditionFailed(resp_body, resp=resp)

        if resp.status == 413:
            if parse_resp:
                resp_body = self._parse_resp(resp_body)
            if self.is_absolute_limit(resp, resp_body):
                raise exceptions.OverLimit(resp_body, resp=resp)
            else:
                raise exceptions.RateLimitExceeded(resp_body, resp=resp)

        if resp.status == 415:
            if parse_resp:
                resp_body = self._parse_resp(resp_body)
            raise exceptions.InvalidContentType(resp_body, resp=resp)

        if resp.status == 422:
            if parse_resp:
                resp_body = self._parse_resp(resp_body)
            raise exceptions.UnprocessableEntity(resp_body, resp=resp)

        if resp.status in (500, 501):
            message = resp_body
            if parse_resp:
                try:
                    resp_body = self._parse_resp(resp_body)
                except ValueError:
                    # If response body is a non-json string message.
                    # Use resp_body as is and raise InvalidResponseBody
                    # exception.
                    raise exceptions.InvalidHTTPResponseBody(message)
                else:
                    if isinstance(resp_body, dict):
                        # I'm seeing both computeFault
                        # and cloudServersFault come back.
                        # Will file a bug to fix, but leave as is for now.
                        if 'cloudServersFault' in resp_body:
                            message = resp_body['cloudServersFault']['message']
                        elif 'computeFault' in resp_body:
                            message = resp_body['computeFault']['message']
                        elif 'error' in resp_body:
                            message = resp_body['error']['message']
                        elif 'message' in resp_body:
                            message = resp_body['message']
                    else:
                        message = resp_body

            if resp.status == 501:
                raise exceptions.NotImplemented(resp_body, resp=resp,
                                                message=message)
            else:
                raise exceptions.ServerFault(resp_body, resp=resp,
                                             message=message)

        if resp.status >= 400:
            raise exceptions.UnexpectedResponseCode(str(resp.status),
                                                    resp=resp)

    def is_absolute_limit(self, resp, resp_body):
        if (not isinstance(resp_body, collections.Mapping) or
                'retry-after' not in resp):
            return True
        return 'exceed' in resp_body.get('message', 'blabla')

    def wait_for_resource_deletion(self, id, *args, **kwargs):
        """Waits for a resource to be deleted

        This method will loop over is_resource_deleted until either
        is_resource_deleted returns True or the build timeout is reached. This
        depends on is_resource_deleted being implemented

        :param str id: The id of the resource to check
        :raises TimeoutException: If the build_timeout has elapsed and the
                                  resource still hasn't been deleted
        """
        start_time = int(time.time())
        while True:
            if self.is_resource_deleted(id, *args, **kwargs):
                return
            if int(time.time()) - start_time >= self.build_timeout:
                message = ('Failed to delete %(resource_type)s %(id)s within '
                           'the required time (%(timeout)s s).' %
                           {'resource_type': self.resource_type, 'id': id,
                            'timeout': self.build_timeout})
                caller = test_utils.find_test_caller()
                if caller:
                    message = '(%s) %s' % (caller, message)
                raise exceptions.TimeoutException(message)
            time.sleep(self.build_interval)

    def wait_for_resource_activation(self, id):
        """Waits for a resource to become active

        This method will loop over is_resource_active until either
        is_resource_active returns True or the build timeout is reached. This
        depends on is_resource_active being implemented

        :param str id: The id of the resource to check
        :raises TimeoutException: If the build_timeout has elapsed and the
                                  resource still hasn't been active
        """
        start_time = int(time.time())
        while True:
            if self.is_resource_active(id):
                return
            if int(time.time()) - start_time >= self.build_timeout:
                message = ('Failed to reach active state %(resource_type)s '
                           '%(id)s within the required time (%(timeout)s s).' %
                           {'resource_type': self.resource_type, 'id': id,
                            'timeout': self.build_timeout})
                caller = test_utils.find_test_caller()
                if caller:
                    message = '(%s) %s' % (caller, message)
                raise exceptions.TimeoutException(message)
            time.sleep(self.build_interval)

    def is_resource_deleted(self, id):
        """Subclasses override with specific deletion detection."""
        message = ('"%s" does not implement is_resource_deleted'
                   % self.__class__.__name__)
        raise NotImplementedError(message)

    def is_resource_active(self, id):
        """Subclasses override with specific active detection."""
        message = ('"%s" does not implement is_resource_active'
                   % self.__class__.__name__)
        raise NotImplementedError(message)

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'resource'

    @classmethod
    def validate_response(cls, schema, resp, body):
        # Only check the response if the status code is a success code
        # TODO(cyeoh): Eventually we should be able to verify that a failure
        # code if it exists is something that we expect. This is explicitly
        # declared in the V3 API and so we should be able to export this in
        # the response schema. For now we'll ignore it.
        if resp.status in HTTP_SUCCESS + HTTP_REDIRECTION:
            cls.expected_success(schema['status_code'], resp.status)

            # Check the body of a response
            body_schema = schema.get('response_body')
            if body_schema:
                try:
                    jsonschema.validate(body, body_schema,
                                        cls=JSONSCHEMA_VALIDATOR,
                                        format_checker=FORMAT_CHECKER)
                except jsonschema.ValidationError as ex:
                    msg = ("HTTP response body is invalid (%s)" % ex)
                    raise exceptions.InvalidHTTPResponseBody(msg)
            else:
                if body:
                    msg = ("HTTP response body should not exist (%s)" % body)
                    raise exceptions.InvalidHTTPResponseBody(msg)

            # Check the header of a response
            header_schema = schema.get('response_header')
            if header_schema:
                try:
                    jsonschema.validate(resp, header_schema,
                                        cls=JSONSCHEMA_VALIDATOR,
                                        format_checker=FORMAT_CHECKER)
                except jsonschema.ValidationError as ex:
                    msg = ("HTTP response header is invalid (%s)" % ex)
                    raise exceptions.InvalidHTTPResponseHeader(msg)

    def _get_base_version_url(self):
        # TODO(oomichi): This method can be used for auth's replace_version().
        # So it is nice to have common logic for the maintenance.
        endpoint = self.base_url
        url = urllib.parse.urlsplit(endpoint)
        new_path = re.split(r'(^|/)+v\d+(\.\d+)?', url.path)[0]
        url = list(url)
        url[2] = new_path + '/'
        return urllib.parse.urlunsplit(url)


class ResponseBody(dict):
    """Class that wraps an http response and dict body into a single value.

    Callers that receive this object will normally use it as a dict but
    can extract the response if needed.
    """

    def __init__(self, response, body=None):
        body_data = body or {}
        self.update(body_data)
        self.response = response

    def __str__(self):
        body = super(ResponseBody, self).__str__()
        return "response: %s\nBody: %s" % (self.response, body)


class ResponseBodyData(object):
    """Class that wraps an http response and string data into a single value.

    """

    def __init__(self, response, data):
        self.response = response
        self.data = data

    def __str__(self):
        return "response: %s\nBody: %s" % (self.response, self.data)


class ResponseBodyList(list):
    """Class that wraps an http response and list body into a single value.

    Callers that receive this object will normally use it as a list but
    can extract the response if needed.
    """

    def __init__(self, response, body=None):
        body_data = body or []
        self.extend(body_data)
        self.response = response

    def __str__(self):
        body = super(ResponseBodyList, self).__str__()
        return "response: %s\nBody: %s" % (self.response, body)



# Copyright 2016 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.identity.v2 import users_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestUsersClient(base.BaseServiceTest):
    FAKE_USER_INFO = {
        "user": {
            "id": "1",
            "name": "test",
            "email": "john.smith@example.org",
            "enabled": True
        }
    }

    FAKE_LIST_USERS = {
        "users": [
            {
                "id": "1",
                "name": "test",
                "email": "john.smith@example.org",
                "enabled": True
            },
            {
                "id": "2",
                "name": "test2",
                "email": "john.smith@example.org",
                "enabled": True
            }
        ]
    }

    FAKE_USER_EC2_CREDENTIAL_INFO = {
        "credential": {
            'user_id': '9beb0e12f3e5416db8d7cccfc785db3b',
            'access': '79abf59acc77492a86170cbe2f1feafa',
            'secret': 'c4e7d3a691fd4563873d381a40320f46',
            'trust_id': None,
            'tenant_id': '596557269d7b4dd78631a602eb9f151d'
        }
    }

    FAKE_LIST_USER_EC2_CREDENTIALS = {
        "credentials": [
            {
                'user_id': '9beb0e12f3e5416db8d7cccfc785db3b',
                'access': '79abf59acc77492a86170cbe2f1feafa',
                'secret': 'c4e7d3a691fd4563873d381a40320f46',
                'trust_id': None,
                'tenant_id': '596557269d7b4dd78631a602eb9f151d'
            },
            {
                'user_id': '3beb0e12f3e5416db8d7cccfc785de4r',
                'access': '45abf59acc77492a86170cbe2f1fesde',
                'secret': 'g4e7d3a691fd4563873d381a40320e45',
                'trust_id': None,
                'tenant_id': '123557269d7b4dd78631a602eb9f112f'
            }
        ]
    }

    def setUp(self):
        super(TestUsersClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = users_client.UsersClient(fake_auth,
                                               'identity', 'regionOne')

    def _test_create_user(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_user,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_USER_INFO,
            bytes_body,
            name="test",
            email="john.smith@example.org")

    def _test_update_user(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_user,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_USER_INFO,
            bytes_body,
            user_id="1",
            name="test")

    def _test_show_user(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_user,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_USER_INFO,
            bytes_body,
            user_id="1")

    def _test_list_users(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_users,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_USERS,
            bytes_body)

    def _test_update_user_enabled(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_user_enabled,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_USER_INFO,
            bytes_body,
            user_id="1",
            enabled=True)

    def _test_update_user_password(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_user_password,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_USER_INFO,
            bytes_body,
            user_id="1",
            password="pass")

    def _test_update_user_own_password(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_user_own_password,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_USER_INFO,
            bytes_body,
            user_id="1",
            password="pass")

    def _test_create_user_ec2_credential(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_user_ec2_credential,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_USER_EC2_CREDENTIAL_INFO,
            bytes_body,
            user_id="1",
            tenant_id="123")

    def _test_show_user_ec2_credential(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_user_ec2_credential,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_USER_EC2_CREDENTIAL_INFO,
            bytes_body,
            user_id="1",
            access="123")

    def _test_list_user_ec2_credentials(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_user_ec2_credentials,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_USER_EC2_CREDENTIALS,
            bytes_body,
            user_id="1")

    def test_create_user_with_str_body(self):
        self._test_create_user()

    def test_create_user_with_bytes_body(self):
        self._test_create_user(bytes_body=True)

    def test_update_user_with_str_body(self):
        self._test_update_user()

    def test_update_user_with_bytes_body(self):
        self._test_update_user(bytes_body=True)

    def test_show_user_with_str_body(self):
        self._test_show_user()

    def test_show_user_with_bytes_body(self):
        self._test_show_user(bytes_body=True)

    def test_list_users_with_str_body(self):
        self._test_list_users()

    def test_list_users_with_bytes_body(self):
        self._test_list_users(bytes_body=True)

    def test_delete_user(self):
        self.check_service_client_function(
            self.client.delete_user,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            user_id="1",
            status=204)

    def test_update_user_enabled_with_str_body(self):
        self._test_update_user_enabled()

    def test_update_user_enabled_with_bytes_body(self):
        self._test_update_user_enabled(bytes_body=True)

    def test_update_user_password_with_str_body(self):
        self._test_update_user_password()

    def test_update_user_password_with_bytes_body(self):
        self._test_update_user_password(bytes_body=True)

    def test_update_user_own_password_with_str_body(self):
        self._test_update_user_own_password()

    def test_update_user_own_password_with_bytes_body(self):
        self._test_update_user_own_password(bytes_body=True)

    def test_create_user_ec2_credential_with_str_body(self):
        self._test_create_user_ec2_credential()

    def test_create_user_ec2_credential_with_bytes_body(self):
        self._test_create_user_ec2_credential(bytes_body=True)

    def test_show_user_ec2_credential_with_str_body(self):
        self._test_show_user_ec2_credential()

    def test_show_user_ec2_credential_with_bytes_body(self):
        self._test_show_user_ec2_credential(bytes_body=True)

    def test_list_user_ec2_credentials_with_str_body(self):
        self._test_list_user_ec2_credentials()

    def test_list_user_ec2_credentials_with_bytes_body(self):
        self._test_list_user_ec2_credentials(bytes_body=True)

    def test_delete_user_ec2_credential(self):
        self.check_service_client_function(
            self.client.delete_user_ec2_credential,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            user_id="123",
            access="1234",
            status=204)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import api_microversion_fixture
from tempest.common import compute
from tempest.common import waiters
from tempest import config
from tempest.lib.common import api_version_utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
import tempest.test

CONF = config.CONF


class BaseVolumeTest(api_version_utils.BaseMicroversionTest,
                     tempest.test.BaseTestCase):
    """Base test case class for all Cinder API tests."""

    # Set this to True in subclasses to create a default network. See
    # https://bugs.launchpad.net/tempest/+bug/1844568
    create_default_network = False
    credentials = ['primary']

    @classmethod
    def skip_checks(cls):
        super(BaseVolumeTest, cls).skip_checks()

        if not CONF.service_available.cinder:
            skip_msg = ("%s skipped as Cinder is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

        api_version_utils.check_skip_with_microversion(
            cls.min_microversion, cls.max_microversion,
            CONF.volume.min_microversion, CONF.volume.max_microversion)

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources(
            network=cls.create_default_network,
            subnet=cls.create_default_network)
        super(BaseVolumeTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(BaseVolumeTest, cls).setup_clients()
        cls.servers_client = cls.os_primary.servers_client

        if CONF.service_available.glance:
            cls.images_client = cls.os_primary.image_client_v2

        cls.backups_client = cls.os_primary.backups_client_latest
        cls.volumes_client = cls.os_primary.volumes_client_latest
        cls.messages_client = cls.os_primary.volume_messages_client_latest
        cls.versions_client = cls.os_primary.volume_versions_client_latest
        cls.groups_client = cls.os_primary.groups_client_latest
        cls.group_snapshots_client = (
            cls.os_primary.group_snapshots_client_latest)
        cls.snapshots_client = cls.os_primary.snapshots_client_latest
        cls.volumes_extension_client =\
            cls.os_primary.volumes_extension_client_latest
        cls.availability_zone_client = (
            cls.os_primary.volume_availability_zone_client_latest)
        cls.volume_limits_client = cls.os_primary.volume_limits_client_latest

    def setUp(self):
        super(BaseVolumeTest, self).setUp()
        self.useFixture(api_microversion_fixture.APIMicroversionFixture(
            self.request_microversion))

    @classmethod
    def resource_setup(cls):
        super(BaseVolumeTest, cls).resource_setup()
        cls.request_microversion = (
            api_version_utils.select_request_microversion(
                cls.min_microversion,
                CONF.volume.min_microversion))

        cls.image_ref = CONF.compute.image_ref
        cls.flavor_ref = CONF.compute.flavor_ref
        cls.build_interval = CONF.volume.build_interval
        cls.build_timeout = CONF.volume.build_timeout

    @classmethod
    def create_volume(cls, wait_until='available', **kwargs):
        """Wrapper utility that returns a test volume.

           :param wait_until: wait till volume status.
        """
        if 'size' not in kwargs:
            kwargs['size'] = CONF.volume.volume_size

        if 'imageRef' in kwargs:
            image = cls.images_client.show_image(kwargs['imageRef'])
            min_disk = image['min_disk']
            kwargs['size'] = max(kwargs['size'], min_disk)

        if 'name' not in kwargs:
            name = data_utils.rand_name(cls.__name__ + '-Volume')
            kwargs['name'] = name

        if CONF.volume.volume_type and 'volume_type' not in kwargs:
            # If volume_type is not provided in config then no need to
            # add a volume type and
            # if volume_type has already been added by child class then
            # no need to override.
            kwargs['volume_type'] = CONF.volume.volume_type

        if CONF.compute.compute_volume_common_az:
            kwargs.setdefault('availability_zone',
                              CONF.compute.compute_volume_common_az)

        volume = cls.volumes_client.create_volume(**kwargs)['volume']
        cls.addClassResourceCleanup(test_utils.call_and_ignore_notfound_exc,
                                    cls.delete_volume, cls.volumes_client,
                                    volume['id'])
        waiters.wait_for_volume_resource_status(cls.volumes_client,
                                                volume['id'], wait_until)
        return volume

    @classmethod
    def create_snapshot(cls, volume_id=1, **kwargs):
        """Wrapper utility that returns a test snapshot."""
        if 'name' not in kwargs:
            name = data_utils.rand_name(cls.__name__ + '-Snapshot')
            kwargs['name'] = name

        snapshot = cls.snapshots_client.create_snapshot(
            volume_id=volume_id, **kwargs)['snapshot']
        cls.addClassResourceCleanup(test_utils.call_and_ignore_notfound_exc,
                                    cls.delete_snapshot, snapshot['id'])
        waiters.wait_for_volume_resource_status(cls.snapshots_client,
                                                snapshot['id'], 'available')
        return snapshot

    def create_backup(self, volume_id, backup_client=None, **kwargs):
        """Wrapper utility that returns a test backup."""
        if backup_client is None:
            backup_client = self.backups_client
        if 'name' not in kwargs:
            name = data_utils.rand_name(self.__class__.__name__ + '-Backup')
            kwargs['name'] = name

        backup = backup_client.create_backup(
            volume_id=volume_id, **kwargs)['backup']
        self.addCleanup(backup_client.delete_backup, backup['id'])
        waiters.wait_for_volume_resource_status(backup_client, backup['id'],
                                                'available')
        return backup

    # NOTE(afazekas): these create_* and clean_* could be defined
    # only in a single location in the source, and could be more general.

    @staticmethod
    def delete_volume(client, volume_id):
        """Delete volume by the given client"""
        client.delete_volume(volume_id)
        client.wait_for_resource_deletion(volume_id)

    @classmethod
    def delete_snapshot(cls, snapshot_id, snapshots_client=None):
        """Delete snapshot by the given client"""
        if snapshots_client is None:
            snapshots_client = cls.snapshots_client
        snapshots_client.delete_snapshot(snapshot_id)
        snapshots_client.wait_for_resource_deletion(snapshot_id)

    def attach_volume(self, server_id, volume_id):
        """Attach a volume to a server"""
        self.servers_client.attach_volume(
            server_id, volumeId=volume_id,
            device='/dev/%s' % CONF.compute.volume_device_name)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume_id, 'in-use')
        self.addCleanup(waiters.wait_for_volume_resource_status,
                        self.volumes_client, volume_id, 'available')
        self.addCleanup(self.servers_client.detach_volume, server_id,
                        volume_id)

    def create_server(self, wait_until='ACTIVE', **kwargs):
        name = kwargs.pop(
            'name',
            data_utils.rand_name(self.__class__.__name__ + '-instance'))

        tenant_network = self.get_tenant_network()
        body, _ = compute.create_test_server(
            self.os_primary,
            tenant_network=tenant_network,
            name=name,
            wait_until=wait_until,
            **kwargs)

        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        waiters.wait_for_server_termination,
                        self.servers_client, body['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.servers_client.delete_server, body['id'])
        return body

    def create_group(self, **kwargs):
        if 'name' not in kwargs:
            kwargs['name'] = data_utils.rand_name(
                self.__class__.__name__ + '-Group')

        group = self.groups_client.create_group(**kwargs)['group']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.delete_group, group['id'])
        waiters.wait_for_volume_resource_status(
            self.groups_client, group['id'], 'available')
        return group

    def delete_group(self, group_id, delete_volumes=True):
        group_vols = []
        if delete_volumes:
            vols = self.volumes_client.list_volumes(detail=True)['volumes']
            for vol in vols:
                if vol['group_id'] == group_id:
                    group_vols.append(vol['id'])
        self.groups_client.delete_group(group_id, delete_volumes)
        for vol in group_vols:
            self.volumes_client.wait_for_resource_deletion(vol)
        self.groups_client.wait_for_resource_deletion(group_id)


class BaseVolumeAdminTest(BaseVolumeTest):
    """Base test case class for all Volume Admin API tests."""

    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super(BaseVolumeAdminTest, cls).setup_clients()

        cls.admin_volume_qos_client = cls.os_admin.volume_qos_client_latest
        cls.admin_volume_services_client = \
            cls.os_admin.volume_services_client_latest
        cls.admin_volume_types_client = cls.os_admin.volume_types_client_latest
        cls.admin_volume_manage_client = (
            cls.os_admin.volume_manage_client_latest)
        cls.admin_volume_client = cls.os_admin.volumes_client_latest
        cls.admin_groups_client = cls.os_admin.groups_client_latest
        cls.admin_messages_client = cls.os_admin.volume_messages_client_latest
        cls.admin_group_snapshots_client = \
            cls.os_admin.group_snapshots_client_latest
        cls.admin_group_types_client = cls.os_admin.group_types_client_latest
        cls.admin_hosts_client = cls.os_admin.volume_hosts_client_latest
        cls.admin_snapshot_manage_client = \
            cls.os_admin.snapshot_manage_client_latest
        cls.admin_snapshots_client = cls.os_admin.snapshots_client_latest
        cls.admin_backups_client = cls.os_admin.backups_client_latest
        cls.admin_encryption_types_client = \
            cls.os_admin.encryption_types_client_latest
        cls.admin_quota_classes_client = \
            cls.os_admin.volume_quota_classes_client_latest
        cls.admin_quotas_client = cls.os_admin.volume_quotas_client_latest
        cls.admin_volume_limits_client = (
            cls.os_admin.volume_limits_client_latest)
        cls.admin_capabilities_client = \
            cls.os_admin.volume_capabilities_client_latest
        cls.admin_scheduler_stats_client = \
            cls.os_admin.volume_scheduler_stats_client_latest

    @classmethod
    def create_test_qos_specs(cls, name=None, consumer=None, **kwargs):
        """create a test Qos-Specs."""
        name = name or data_utils.rand_name(cls.__name__ + '-QoS')
        consumer = consumer or 'front-end'
        qos_specs = cls.admin_volume_qos_client.create_qos(
            name=name, consumer=consumer, **kwargs)['qos_specs']
        cls.addClassResourceCleanup(cls.clear_qos_spec, qos_specs['id'])
        return qos_specs

    @classmethod
    def create_volume_type(cls, name=None, **kwargs):
        """Create a test volume-type"""
        name = name or data_utils.rand_name(cls.__name__ + '-volume-type')
        volume_type = cls.admin_volume_types_client.create_volume_type(
            name=name, **kwargs)['volume_type']
        cls.addClassResourceCleanup(cls.clear_volume_type, volume_type['id'])
        return volume_type

    def create_encryption_type(self, type_id=None, provider=None,
                               key_size=None, cipher=None,
                               control_location=None):
        if not type_id:
            volume_type = self.create_volume_type()
            type_id = volume_type['id']
        self.admin_encryption_types_client.create_encryption_type(
            type_id, provider=provider, key_size=key_size, cipher=cipher,
            control_location=control_location)

    def create_encrypted_volume(self, encryption_provider, key_size=256,
                                cipher='aes-xts-plain64',
                                control_location='front-end'):
        volume_type = self.create_volume_type()
        self.create_encryption_type(type_id=volume_type['id'],
                                    provider=encryption_provider,
                                    key_size=key_size,
                                    cipher=cipher,
                                    control_location=control_location)
        return self.create_volume(volume_type=volume_type['name'])

    def create_group_type(self, name=None, **kwargs):
        """Create a test group-type"""
        name = name or data_utils.rand_name(
            self.__class__.__name__ + '-group-type')
        group_type = self.admin_group_types_client.create_group_type(
            name=name, **kwargs)['group_type']
        self.addCleanup(self.admin_group_types_client.delete_group_type,
                        group_type['id'])
        return group_type

    @classmethod
    def clear_qos_spec(cls, qos_id):
        test_utils.call_and_ignore_notfound_exc(
            cls.admin_volume_qos_client.delete_qos, qos_id)

        test_utils.call_and_ignore_notfound_exc(
            cls.admin_volume_qos_client.wait_for_resource_deletion, qos_id)

    @classmethod
    def clear_volume_type(cls, vol_type_id):
        test_utils.call_and_ignore_notfound_exc(
            cls.admin_volume_types_client.delete_volume_type, vol_type_id)

        test_utils.call_and_ignore_notfound_exc(
            cls.admin_volume_types_client.wait_for_resource_deletion,
            vol_type_id)



# Copyright 2013 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import tenant_usages
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class TenantUsagesClient(base_compute_client.BaseComputeClient):

    def list_tenant_usages(self, **params):
        """List Tenant Usage For All Tenants.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-tenant-usage-statistics-for-all-tenants
        """
        url = 'os-simple-tenant-usage'
        if params:
            url += '?%s' % urllib.urlencode(params)

        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(tenant_usages.list_tenant_usage, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_tenant_usage(self, tenant_id, **params):
        """Show Usage Details For Tenant.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#show-usage-statistics-for-tenant
        """
        url = 'os-simple-tenant-usage/%s' % tenant_id
        if params:
            url += '?%s' % urllib.urlencode(params)

        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(tenant_usages.get_tenant_usage, resp, body)
        return rest_client.ResponseBody(resp, body)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.placement import resource_providers_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestResourceProvidersClient(base.BaseServiceTest):
    FAKE_RESOURCE_PROVIDER_UUID = '3722a86e-a563-11e9-9abb-c3d41b6d3abf'
    FAKE_ROOT_PROVIDER_UUID = '4a6a57c8-a563-11e9-914e-f3e0478fce53'
    FAKE_RESOURCE_PROVIDER = {
        'generation': 0,
        'name': 'Ceph Storage Pool',
        'uuid': FAKE_RESOURCE_PROVIDER_UUID,
        'parent_provider_uuid': FAKE_ROOT_PROVIDER_UUID,
        'root_provider_uuid': FAKE_ROOT_PROVIDER_UUID
    }

    FAKE_RESOURCE_PROVIDERS = {
        'resource_providers': [FAKE_RESOURCE_PROVIDER]
    }

    FAKE_RESOURCE_PROVIDER_INVENTORIES = {
        'inventories': {
            'DISK_GB': {
                'allocation_ratio': 1.0,
                'max_unit': 35,
                'min_unit': 1,
                'reserved': 0,
                'step_size': 1,
                'total': 35
            }
        },
        'resource_provider_generation': 7
    }

    FAKE_AGGREGATE_UUID = '1166be40-a567-11e9-9f2a-53827f9311fa'
    FAKE_RESOURCE_PROVIDER_AGGREGATES = {
        'aggregates': [FAKE_AGGREGATE_UUID]
    }
    FAKE_RESOURCE_UPDATE_INVENTORIES_RESPONSE = {
        "inventories": {
            "MEMORY_MB": {
                "allocation_ratio": 2.0,
                "max_unit": 16,
                "min_unit": 1,
                "reserved": 0,
                "step_size": 4,
                "total": 128
            },
            "VCPU": {
                "allocation_ratio": 10.0,
                "max_unit": 2147483647,
                "min_unit": 1,
                "reserved": 2,
                "step_size": 1,
                "total": 64
            }
        },
        "resource_provider_generation": 2
    }
    FAKE_RESOURCE_UPDATE_INVENTORIES_REQUEST = {
        "inventories": {
            "MEMORY_MB": {
                "allocation_ratio": 2.0,
                "max_unit": 16,
                "step_size": 4,
                "total": 128
            },
            "VCPU": {
                "allocation_ratio": 10.0,
                "reserved": 2,
                "total": 64
            }
        },
        "resource_provider_generation": 1
    }

    def setUp(self):
        super(TestResourceProvidersClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = resource_providers_client.ResourceProvidersClient(
            fake_auth, 'placement', 'regionOne')

    def _test_list_resource_providers(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_resource_providers,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_RESOURCE_PROVIDERS,
            to_utf=bytes_body,
            status=200
        )

    def test_list_resource_providers_with_bytes_body(self):
        self._test_list_resource_providers()

    def test_list_resource_providers_with_str_body(self):
        self._test_list_resource_providers(bytes_body=True)

    def _test_show_resource_provider(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_resource_provider,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_RESOURCE_PROVIDER,
            to_utf=bytes_body,
            status=200,
            rp_uuid=self.FAKE_RESOURCE_PROVIDER_UUID
        )

    def test_show_resource_provider_with_str_body(self):
        self._test_show_resource_provider()

    def test_show_resource_provider_with_bytes_body(self):
        self._test_show_resource_provider(bytes_body=True)

    def _test_list_resource_provider_inventories(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_resource_provider_inventories,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_RESOURCE_PROVIDER_INVENTORIES,
            to_utf=bytes_body,
            status=200,
            rp_uuid=self.FAKE_RESOURCE_PROVIDER_UUID
        )

    def test_list_resource_provider_inventories_with_str_body(self):
        self._test_list_resource_provider_inventories()

    def test_list_resource_provider_inventories_with_bytes_body(self):
        self._test_list_resource_provider_inventories(bytes_body=True)

    def _test_update_resource_providers_inventories(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_resource_providers_inventories,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_RESOURCE_UPDATE_INVENTORIES_RESPONSE,
            to_utf=bytes_body,
            status=200,
            rp_uuid=self.FAKE_RESOURCE_PROVIDER_UUID,
            **self.FAKE_RESOURCE_UPDATE_INVENTORIES_REQUEST
        )

    def test_update_resource_providers_inventories_with_str_body(self):
        self._test_update_resource_providers_inventories()

    def test_update_resource_providers_inventories_with_bytes_body(self):
        self._test_update_resource_providers_inventories(bytes_body=True)

    def test_delete_resource_providers_inventories(self):
        self.check_service_client_function(
            self.client.delete_resource_providers_inventories,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            rp_uuid=self.FAKE_RESOURCE_PROVIDER_UUID,
        )

    def _test_list_resource_provider_aggregates(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_resource_provider_aggregates,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_RESOURCE_PROVIDER_AGGREGATES,
            to_utf=bytes_body,
            status=200,
            rp_uuid=self.FAKE_RESOURCE_PROVIDER_UUID
        )

    def test_list_resource_provider_aggregates_with_str_body(self):
        self._test_list_resource_provider_aggregates()

    def test_list_resource_provider_aggregates_with_bytes_body(self):
        self._test_list_resource_provider_aggregates(bytes_body=True)



#!/usr/bin/env python

# Copyright 2015 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Utility for creating ``accounts.yaml`` file for concurrent test runs.
Creates one primary user, one alt user, one swift admin, one stack owner
and one admin (optionally) for each concurrent thread. The utility creates
user for each tenant. The ``accounts.yaml`` file will be valid and contain
credentials for created users, so each user will be in separate tenant and
have the username, tenant_name, password and roles.

**Usage:** ``tempest account-generator [-h] [OPTIONS] accounts_file.yaml``

Positional Arguments
--------------------
``accounts_file.yaml`` (Required) Provide an output accounts yaml file. Utility
creates a .yaml file in the directory where the command is ran. The appropriate
name for the file is *accounts.yaml* and it should be placed in *tempest/etc*
directory.

Authentication
--------------

Account generator creates users and tenants so it needs the admin credentials
of your cloud to operate properly. The corresponding info can be given either
through CLI options or environment variables.

You're probably familiar with these, but just to remind:

======== ============================ ====================
Param    CLI                          Environment Variable
======== ============================ ====================
Username ``--os-username``            OS_USERNAME
Password ``--os-password``            OS_PASSWORD
Project  ``--os-project-name``        OS_PROJECT_NAME
Domain   ``--os-domain-name``         OS_DOMAIN_NAME
======== ============================ ====================

Optional Arguments
------------------
* ``-h, --help`` (Optional) Shows help message with the description of
  utility and its arguments, and exits.

* ``-c, --config-file /etc/tempest.conf`` (Optional) Path
  to tempest config file. If not specified, it searches for tempest.conf in
  these locations:

  - ./etc/
  - /etc/tempest
  - ~/.tempest/
  - ~/
  - /etc/

* ``--os-username <auth-user-name>`` (Optional) Name used for authentication
  with the OpenStack Identity service. Defaults to env[OS_USERNAME]. Note: User
  should have permissions to create new user accounts and tenants.

* ``--os-password <auth-password>`` (Optional) Password used for authentication
  with the OpenStack Identity service. Defaults to env[OS_PASSWORD].

* ``--os-project-name <auth-project-name>`` (Optional) Project to request
  authorization on. Defaults to env[OS_PROJECT_NAME].

* ``--os-domain-name <auth-domain-name>`` (Optional) Domain the user and
  project belong to. Defaults to env[OS_DOMAIN_NAME].

* ``--tag TAG`` (Optional) Resources tag. Each created resource (user, project)
  will have the prefix with the given TAG in its name. Using tag is recommended
  for the further using, cleaning resources.

* ``-r, --concurrency CONCURRENCY`` (Optional) Concurrency count
  (default: 1). The number of accounts required can be estimated as
  CONCURRENCY x 2. Each user provided in *accounts.yaml* file will be in
  a different tenant. This is required to provide isolation between test for
  running in parallel.

* ``--with-admin`` (Optional) Creates admin for each concurrent group
  (default: False).

* ``-i, --identity-version VERSION`` (Optional) Provisions accounts
  using the specified version of the identity API. (default: '3').

To see help on specific argument, please do: ``tempest account-generator
[OPTIONS] <accounts_file.yaml> -h``.
"""

import argparse
import os
import traceback

from cliff import command
from oslo_log import log as logging
import yaml

from tempest.common import credentials_factory
from tempest import config
from tempest.lib.common import dynamic_creds


LOG = None
CONF = config.CONF
DESCRIPTION = ('Create accounts.yaml file for concurrent test runs.%s'
               'One primary user, one alt user, '
               'one swift admin, one stack owner '
               'and one admin (optionally) will be created '
               'for each concurrent thread.' % os.linesep)


def setup_logging():
    global LOG
    logging.setup(CONF, __name__)
    LOG = logging.getLogger(__name__)


def get_credential_provider(opts):
    identity_version = "".join(['v', str(opts.identity_version)])
    # NOTE(andreaf) For now tempest.conf controls whether resources will
    # actually be created. Once we remove the dependency from tempest.conf
    # we will need extra CLI option(s) to control this.
    network_resources = {'router': True,
                         'network': True,
                         'subnet': True,
                         'dhcp': True}
    admin_creds_dict = {'username': opts.os_username,
                        'password': opts.os_password}
    _project_name = opts.os_project_name
    if opts.identity_version == 3:
        admin_creds_dict['project_name'] = _project_name
        admin_creds_dict['domain_name'] = opts.os_domain_name or 'Default'
    elif opts.identity_version == 2:
        admin_creds_dict['tenant_name'] = _project_name
    admin_creds = credentials_factory.get_credentials(
        fill_in=False, identity_version=identity_version, **admin_creds_dict)
    return dynamic_creds.DynamicCredentialProvider(
        name=opts.tag,
        network_resources=network_resources,
        **credentials_factory.get_dynamic_provider_params(
            identity_version, admin_creds=admin_creds))


def generate_resources(cred_provider, admin):
    # Create the list of resources to be provisioned for each process
    # NOTE(andreaf) get_credentials expects a string for types or a list for
    # roles. Adding all required inputs to the spec list.
    spec = ['primary', 'alt']
    if CONF.service_available.swift:
        spec.append([CONF.object_storage.operator_role])
        spec.append([CONF.object_storage.reseller_admin_role])
    if admin:
        spec.append('admin')
    resources = []
    for cred_type in spec:
        resources.append((cred_type, cred_provider.get_credentials(
            credential_type=cred_type)))
    return resources


def dump_accounts(resources, identity_version, account_file):
    accounts = []
    for resource in resources:
        cred_type, test_resource = resource
        account = {
            'username': test_resource.username,
            'password': test_resource.password
        }
        if identity_version == 3:
            account['project_name'] = test_resource.project_name
            account['domain_name'] = test_resource.domain_name
        else:
            account['project_name'] = test_resource.tenant_name

        # If the spec includes 'admin' credentials are defined via type,
        # else they are defined via list of roles.
        if cred_type == 'admin':
            account['types'] = [cred_type]
        elif cred_type not in ['primary', 'alt']:
            account['roles'] = cred_type

        if test_resource.network:
            account['resources'] = {}
            account['resources']['network'] = test_resource.network['name']
        accounts.append(account)
    if os.path.exists(account_file):
        os.rename(account_file, '.'.join((account_file, 'bak')))
    with open(account_file, 'w') as f:
        yaml.safe_dump(accounts, f, default_flow_style=False)
    LOG.info('%s generated successfully!', account_file)


def positive_int(number):
    number = int(number)
    if number <= 0:
        raise argparse.ArgumentTypeError("Concurrency value should be a "
                                         "positive number")
    return number


def _parser_add_args(parser):
    parser.add_argument('-c', '--config-file',
                        metavar='/etc/tempest.conf',
                        help='path to tempest config file')
    parser.add_argument('--os-username',
                        metavar='<auth-user-name>',
                        default=os.environ.get('OS_USERNAME'),
                        help='User should have permissions '
                             'to create new user accounts and '
                             'tenants. Defaults to env[OS_USERNAME].')
    parser.add_argument('--os-password',
                        metavar='<auth-password>',
                        default=os.environ.get('OS_PASSWORD'),
                        help='Defaults to env[OS_PASSWORD].')
    parser.add_argument('--os-project-name',
                        metavar='<auth-project-name>',
                        default=os.environ.get('OS_PROJECT_NAME'),
                        help='Defaults to env[OS_PROJECT_NAME].')
    parser.add_argument('--os-domain-name',
                        metavar='<auth-domain-name>',
                        default=os.environ.get('OS_DOMAIN_NAME'),
                        help='Defaults to env[OS_DOMAIN_NAME].')
    parser.add_argument('--tag',
                        default='',
                        required=False,
                        dest='tag',
                        help='Resources tag')
    parser.add_argument('-r', '--concurrency',
                        default=1,
                        type=positive_int,
                        required=False,
                        dest='concurrency',
                        help='Concurrency count')
    parser.add_argument('--with-admin',
                        action='store_true',
                        dest='admin',
                        help='Creates admin for each concurrent group')
    parser.add_argument('-i', '--identity-version',
                        default=3,
                        choices=[2, 3],
                        type=int,
                        required=False,
                        dest='identity_version',
                        help='Version of the Identity API to use')
    parser.add_argument('accounts',
                        metavar='accounts_file.yaml',
                        help='Output accounts yaml file')


class TempestAccountGenerator(command.Command):

    def get_parser(self, prog_name):
        parser = super(TempestAccountGenerator, self).get_parser(prog_name)
        _parser_add_args(parser)
        return parser

    def take_action(self, parsed_args):
        try:
            if parsed_args.config_file:
                config.CONF.set_config_path(parsed_args.config_file)
            setup_logging()
            resources = []
            for _ in range(parsed_args.concurrency):
                # Use N different cred_providers to obtain different
                # sets of creds
                cred_provider = get_credential_provider(parsed_args)
                resources.extend(generate_resources(cred_provider,
                                                    parsed_args.admin))
            dump_accounts(resources, parsed_args.identity_version,
                          parsed_args.accounts)

        except Exception:
            LOG.exception("Failure generating test accounts.")
            traceback.print_exc()
            raise

    def get_description(self):
        return DESCRIPTION



# Copyright 2014 Hewlett-Packard Development Company, L.P
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class UsersV3TestJSON(base.BaseIdentityV3AdminTest):
    """Test listing keystone users"""

    def _list_users_with_params(self, params, key, expected, not_expected):
        # Helper method to list users filtered with params and
        # assert the response based on expected and not_expected
        # expected: user expected in the list response
        # not_expected: user, which should not be present in list response
        body = self.users_client.list_users(**params)['users']
        self.assertIn(expected[key], map(lambda x: x[key], body))
        self.assertNotIn(not_expected[key],
                         map(lambda x: x[key], body))

    @classmethod
    def skip_checks(cls):
        super(UsersV3TestJSON, cls).skip_checks()
        if CONF.identity_feature_enabled.immutable_user_source:
            raise cls.skipException('Skipped because environment has an '
                                    'immutable user source and solely '
                                    'provides read-only access to users.')

    @classmethod
    def resource_setup(cls):
        super(UsersV3TestJSON, cls).resource_setup()
        alt_user = data_utils.rand_name('test_user')
        alt_password = data_utils.rand_password()
        cls.alt_email = alt_user + '@testmail.tm'
        # Create a domain
        cls.domain = cls.create_domain()
        # Create user with Domain
        cls.users = list()
        u1_name = data_utils.rand_name('test_user')
        cls.domain_enabled_user = cls.users_client.create_user(
            name=u1_name, password=alt_password,
            email=cls.alt_email, domain_id=cls.domain['id'])['user']
        cls.addClassResourceCleanup(cls.users_client.delete_user,
                                    cls.domain_enabled_user['id'])
        cls.users.append(cls.domain_enabled_user)
        # Create default not enabled user
        u2_name = data_utils.rand_name('test_user')
        cls.non_domain_enabled_user = cls.users_client.create_user(
            name=u2_name, password=alt_password,
            email=cls.alt_email, enabled=False)['user']
        cls.addClassResourceCleanup(cls.users_client.delete_user,
                                    cls.non_domain_enabled_user['id'])
        cls.users.append(cls.non_domain_enabled_user)

    @decorators.idempotent_id('08f9aabb-dcfe-41d0-8172-82b5fa0bd73d')
    def test_list_user_domains(self):
        """List users with domain"""
        params = {'domain_id': self.domain['id']}
        self._list_users_with_params(params, 'domain_id',
                                     self.domain_enabled_user,
                                     self.non_domain_enabled_user)

    @decorators.idempotent_id('bff8bf2f-9408-4ef5-b63a-753c8c2124eb')
    def test_list_users_with_not_enabled(self):
        """List the users with not enabled"""
        params = {'enabled': False}
        self._list_users_with_params(params, 'enabled',
                                     self.non_domain_enabled_user,
                                     self.domain_enabled_user)

    @decorators.idempotent_id('c285bb37-7325-4c02-bff3-3da5d946d683')
    def test_list_users_with_name(self):
        """List users with name"""
        params = {'name': self.domain_enabled_user['name']}
        # When domain specific drivers are enabled the operations
        # of listing all users and listing all groups are not supported,
        # they need a domain filter to be specified
        if CONF.identity_feature_enabled.domain_specific_drivers:
            params['domain_id'] = self.domain_enabled_user['domain_id']
        self._list_users_with_params(params, 'name',
                                     self.domain_enabled_user,
                                     self.non_domain_enabled_user)

    @decorators.idempotent_id('b30d4651-a2ea-4666-8551-0c0e49692635')
    def test_list_users(self):
        """List users"""
        # When domain specific drivers are enabled the operations
        # of listing all users and listing all groups are not supported,
        # they need a domain filter to be specified
        if CONF.identity_feature_enabled.domain_specific_drivers:
            body_enabled_user = self.users_client.list_users(
                domain_id=self.domain_enabled_user['domain_id'])['users']
            body_non_enabled_user = self.users_client.list_users(
                domain_id=self.non_domain_enabled_user['domain_id'])['users']
            body = (body_enabled_user + body_non_enabled_user)
        else:
            body = self.users_client.list_users()['users']

        fetched_ids = [u['id'] for u in body]
        missing_users = [u['id'] for u in self.users
                         if u['id'] not in fetched_ids]
        self.assertEmpty(missing_users,
                         "Failed to find user %s in fetched list" %
                         ', '.join(m_user for m_user in missing_users))

    @decorators.idempotent_id('b4baa3ae-ac00-4b4e-9e27-80deaad7771f')
    def test_get_user(self):
        """Get a user detail"""
        user = self.users_client.show_user(self.users[0]['id'])['user']
        self.assertEqual(self.users[0]['id'], user['id'])
        self.assertEqual(self.users[0]['name'], user['name'])
        self.assertEqual(self.alt_email, user['email'])
        self.assertEqual(self.domain['id'], user['domain_id'])



# Copyright 2013 Cloudwatt
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.object_storage import base
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

QUOTA_BYTES = 10
QUOTA_COUNT = 3


class ContainerQuotasTest(base.BaseObjectTest):
    """Attempts to test the perfect behavior of quotas in a container."""

    def setUp(self):
        """Creates and sets a container with quotas.

        Quotas are set by adding meta values to the container,
        and are validated when set:

        - X-Container-Meta-Quota-Bytes:
                     Maximum size of the container, in bytes.
        - X-Container-Meta-Quota-Count:
                     Maximum object count of the container.
        """
        super(ContainerQuotasTest, self).setUp()
        self.container_name = self.create_container()
        metadata = {"quota-bytes": str(QUOTA_BYTES),
                    "quota-count": str(QUOTA_COUNT), }
        self.container_client.create_update_or_delete_container_metadata(
            self.container_name, create_update_metadata=metadata)

    def tearDown(self):
        """Cleans the container of any object after each test."""
        self.delete_containers()
        super(ContainerQuotasTest, self).tearDown()

    @decorators.idempotent_id('9a0fb034-86af-4df0-86fa-f8bd7db21ae0')
    @utils.requires_ext(extension='container_quotas', service='object')
    @decorators.attr(type="smoke")
    def test_upload_valid_object(self):
        """Attempts to uploads an object smaller than the bytes quota."""
        object_name = data_utils.rand_name(name="TestObject")
        data = data_utils.arbitrary_string(QUOTA_BYTES)

        nbefore = self._get_bytes_used()

        resp, _ = self.object_client.create_object(
            self.container_name, object_name, data)
        self.assertHeaders(resp, 'Object', 'PUT')

        nafter = self._get_bytes_used()
        self.assertEqual(nbefore + len(data), nafter)

    @decorators.idempotent_id('22eeeb2b-3668-4160-baef-44790f65a5a0')
    @utils.requires_ext(extension='container_quotas', service='object')
    @decorators.attr(type="smoke")
    def test_upload_large_object(self):
        """Attempts to upload an object larger than the bytes quota."""
        object_name = data_utils.rand_name(name="TestObject")
        data = data_utils.arbitrary_string(QUOTA_BYTES + 1)

        nbefore = self._get_bytes_used()

        self.assertRaises(lib_exc.OverLimit,
                          self.object_client.create_object,
                          self.container_name, object_name, data)

        nafter = self._get_bytes_used()
        self.assertEqual(nbefore, nafter)

    @decorators.idempotent_id('3a387039-697a-44fc-a9c0-935de31f426b')
    @utils.requires_ext(extension='container_quotas', service='object')
    @decorators.attr(type="smoke")
    def test_upload_too_many_objects(self):
        """Attempts to upload many objects that exceeds the count limit."""
        for _ in range(QUOTA_COUNT):
            name = data_utils.rand_name(name="TestObject")
            self.object_client.create_object(self.container_name, name, "")

        nbefore = self._get_object_count()
        self.assertEqual(nbefore, QUOTA_COUNT)

        self.assertRaises(lib_exc.OverLimit,
                          self.object_client.create_object,
                          self.container_name, "OverQuotaObject", "")

        nafter = self._get_object_count()
        self.assertEqual(nbefore, nafter)

    def _get_container_metadata(self):
        resp, _ = self.container_client.list_container_metadata(
            self.container_name)
        return resp

    def _get_object_count(self):
        resp = self._get_container_metadata()
        return int(resp["x-container-object-count"])

    def _get_bytes_used(self):
        resp = self._get_container_metadata()
        return int(resp["x-container-bytes-used"])



# Copyright 2020 Red Hat, Inc
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Utility for content checking of a given dry_run.json file.
"""

import argparse
import json
import sys


def get_parser():
    parser = argparse.ArgumentParser(__doc__)
    parser.add_argument('--is-empty', action="store_true", dest='is_empty',
                        default=False,
                        help="""Are values of a given dry_run.json empty?""")
    parser.add_argument('--file', dest='file', default=None, metavar='PATH',
                        help="A path to a dry_run.json file.")
    return parser


def parse_arguments():
    parser = get_parser()
    args = parser.parse_args()
    if not args.file:
        sys.stderr.write('Path to a dry_run.json must be specified.\n')
        sys.exit(1)
    return args


def load_json(path):
    """Load json content from file addressed by path."""
    try:
        with open(path, 'rb') as json_file:
            json_data = json.load(json_file)
    except Exception as ex:
        sys.exit(ex)
    return json_data


def are_values_empty(dry_run_content):
    """Return true if values of dry_run.json are empty."""
    for value in dry_run_content.values():
        if value:
            return False
    return True


def main():
    args = parse_arguments()
    content = load_json(args.file)
    if args.is_empty:
        if not are_values_empty(content):
            sys.exit(1)


if __name__ == "__main__":
    main()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.volume import base
from tempest.api.volume import test_volumes_extend as extend
from tempest.common import utils
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class EncryptedVolumesExtendAttachedTest(extend.BaseVolumesExtendAttachedTest,
                                         base.BaseVolumeAdminTest):
    """Tests extending the size of an attached encrypted volume."""

    @decorators.idempotent_id('e93243ec-7c37-4b5b-a099-ebf052c13216')
    @testtools.skipUnless(
        CONF.volume_feature_enabled.extend_attached_encrypted_volume,
        "Attached encrypted volume extend is disabled.")
    @utils.services('compute')
    def test_extend_attached_encrypted_volume_luksv1(self):
        volume = self.create_encrypted_volume(encryption_provider="luks")
        self._test_extend_attached_volume(volume)



# Copyright 2018 SUSE Linux GmbH
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

from oslo_utils import timeutils

from tempest.api.identity import base
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class ApplicationCredentialsV3Test(base.BaseApplicationCredentialsV3Test):
    """Test application credentials"""

    def _list_app_creds(self, name=None):
        kwargs = dict(user_id=self.user_id)
        if name:
            kwargs.update(name=name)
        return self.non_admin_app_creds_client.list_application_credentials(
            **kwargs)['application_credentials']

    @decorators.idempotent_id('8080c75c-eddc-4786-941a-c2da7039ae61')
    def test_create_application_credential(self):
        """Test creating application credential"""
        app_cred = self.create_application_credential()

        # Check that the secret appears in the create response
        secret = app_cred['secret']

        # Check that the secret is not retrievable after initial create
        app_cred = self.non_admin_app_creds_client.show_application_credential(
            user_id=self.user_id,
            application_credential_id=app_cred['id']
        )['application_credential']
        self.assertNotIn('secret', app_cred)

        # Check that the application credential is functional
        _, resp = self.non_admin_token.get_token(
            app_cred_id=app_cred['id'],
            app_cred_secret=secret,
            auth_data=True
        )
        self.assertEqual(resp['project']['id'], self.project_id)

    @decorators.idempotent_id('852daf0c-42b5-4239-8466-d193d0543ed3')
    def test_create_application_credential_expires(self):
        """Test creating application credential with expire time"""
        expires_at = timeutils.utcnow() + datetime.timedelta(hours=1)

        app_cred = self.create_application_credential(expires_at=expires_at)

        expires_str = expires_at.isoformat()
        self.assertEqual(expires_str, app_cred['expires_at'])

    @decorators.idempotent_id('529936eb-aa5d-463d-9f79-01c113d3b88f')
    def test_create_application_credential_access_rules(self):
        if not CONF.identity_feature_enabled.access_rules:
            raise self.skipException("Application credential access rules are "
                                     "not available in this environment")
        access_rules = [
            {
                "path": "/v2.1/servers/*/ips",
                "method": "GET",
                "service": "compute"
            }
        ]
        app_cred = self.create_application_credential(
            access_rules=access_rules)
        access_rule_resp = app_cred['access_rules'][0]
        access_rule_resp.pop('id')
        self.assertDictEqual(access_rules[0], access_rule_resp)

    @decorators.idempotent_id('ff0cd457-6224-46e7-b79e-0ada4964a8a6')
    def test_list_application_credentials(self):
        """Test listing application credentials"""
        self.create_application_credential()
        self.create_application_credential()

        app_creds = self._list_app_creds()
        self.assertEqual(2, len(app_creds))

    @decorators.idempotent_id('9bb5e5cc-5250-493a-8869-8b665f6aa5f6')
    def test_query_application_credentials(self):
        """Test listing application credentials filtered by name"""
        self.create_application_credential()
        app_cred_two = self.create_application_credential()
        app_cred_two_name = app_cred_two['name']

        app_creds = self._list_app_creds(name=app_cred_two_name)
        self.assertEqual(1, len(app_creds))
        self.assertEqual(app_cred_two_name, app_creds[0]['name'])



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.lib.services.identity.v3 import policies_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestPoliciesClient(base.BaseServiceTest):
    FAKE_CREATE_POLICY = {
        "policy": {
            "blob": "{'foobar_user': 'role:compute-user'}",
            "project_id": "0426ac1e48f642ef9544c2251e07e261",
            "type": "application/json",
            "user_id": "0ffd248c55b443eaac5253b4e9cbf9b5"
            }
        }

    FAKE_POLICY_INFO = {
        "policy": {
            "blob": {
                "foobar_user": [
                    "role:compute-user"
                    ]
                },
            "id": "717273",
            "links": {
                "self": "http://example.com/identity/v3/policies/717273"
                },
            "project_id": "456789",
            "type": "application/json",
            "user_id": "616263"
            }
        }

    FAKE_ENDPOINT_INFO = {
        "endpoints": [
            {
                "id": "1",
                "interface": "public",
                "links": {
                    "self": "http://example.com/identity/v3/endpoints/1"
                },
                "region": "north",
                "service_id": "9242e05f0c23467bbd1cf1f7a6e5e596",
                "url": "http://example.com/identity/"
            },
            {
                "id": "1",
                "interface": "internal",
                "links": {
                    "self": "http://example.com/identity/v3/endpoints/1"
                },
                "region": "south",
                "service_id": "9242e05f0c23467bbd1cf1f7a6e5e596",
                "url": "http://example.com/identity/"
            }
        ],
        "links": {
            "self": "http://exmp.com/identity/v3/OS-ENDPOINT-POLICY/policies/1"
        }
    }

    FAKE_LIST_POLICIES = {
        "links": {
            "next": None,
            "previous": None,
            "self": "http://example.com/identity/v3/policies"
            },
        "policies": [
            {
                "blob": {
                    "foobar_user": [
                        "role:compute-user"
                        ]
                    },
                "id": "717273",
                "links": {
                    "self": "http://example.com/identity/v3/policies/717273"
                    },
                "project_id": "456789",
                "type": "application/json",
                "user_id": "616263"
                },
            {
                "blob": {
                    "foobar_user": [
                        "role:compute-user"
                        ]
                    },
                "id": "717274",
                "links": {
                    "self": "http://example.com/identity/v3/policies/717274"
                    },
                "project_id": "456789",
                "type": "application/json",
                "user_id": "616263"
                }
            ]
    }
    FAKE_ENDPOINT_ID = "234789"
    FAKE_SERVICE_ID = "556782"
    FAKE_POLICY_ID = "717273"
    FAKE_REGION_ID = "73"

    def setUp(self):
        super(TestPoliciesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = policies_client.PoliciesClient(fake_auth,
                                                     'identity', 'regionOne')

    def _test_create_policy(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_policy,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_POLICY,
            bytes_body,
            status=201)

    def _test_show_policy(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_policy,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_POLICY_INFO,
            bytes_body,
            policy_id="717273")

    def _test_list_policies(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_policies,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_POLICIES,
            bytes_body)

    def _test_update_policy(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_policy,
            'tempest.lib.common.rest_client.RestClient.patch',
            self.FAKE_POLICY_INFO,
            bytes_body,
            policy_id="717273")

    def test_create_policy_with_str_body(self):
        self._test_create_policy()

    def test_create_policy_with_bytes_body(self):
        self._test_create_policy(bytes_body=True)

    def test_show_policy_with_str_body(self):
        self._test_show_policy()

    def test_show_policy_with_bytes_body(self):
        self._test_show_policy(bytes_body=True)

    def test_list_policies_with_str_body(self):
        self._test_list_policies()

    def test_list_policies_with_bytes_body(self):
        self._test_list_policies(bytes_body=True)

    def test_update_policy_with_str_body(self):
        self._test_update_policy()

    def test_update_policy_with_bytes_body(self):
        self._test_update_policy(bytes_body=True)

    def test_delete_policy(self):
        self.check_service_client_function(
            self.client.delete_policy,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            policy_id="717273",
            status=204)

    def test_update_policy_association_for_endpoint(self):
        self.check_service_client_function(
            self.client.update_policy_association_for_endpoint,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            policy_id=self.FAKE_POLICY_ID,
            endpoint_id=self.FAKE_ENDPOINT_ID,
            status=204)

    def test_show_policy_association_for_endpoint(self):
        self.check_service_client_function(
            self.client.show_policy_association_for_endpoint,
            'tempest.lib.common.rest_client.RestClient.get',
            {},
            policy_id=self.FAKE_POLICY_ID,
            endpoint_id=self.FAKE_ENDPOINT_ID,
            status=204)

    def test_delete_policy_association_for_endpoint(self):
        self.check_service_client_function(
            self.client.delete_policy_association_for_endpoint,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            policy_id=self.FAKE_POLICY_ID,
            endpoint_id=self.FAKE_ENDPOINT_ID,
            status=204)

    def test_update_policy_association_for_service(self):
        self.check_service_client_function(
            self.client.update_policy_association_for_service,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            policy_id=self.FAKE_POLICY_ID,
            service_id=self.FAKE_SERVICE_ID,
            status=204)

    def test_show_policy_association_for_service(self):
        self.check_service_client_function(
            self.client.show_policy_association_for_service,
            'tempest.lib.common.rest_client.RestClient.get',
            {},
            policy_id=self.FAKE_POLICY_ID,
            service_id=self.FAKE_SERVICE_ID,
            status=204)

    def test_delete_policy_association_for_service(self):
        self.check_service_client_function(
            self.client.delete_policy_association_for_service,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            policy_id=self.FAKE_POLICY_ID,
            service_id=self.FAKE_SERVICE_ID,
            status=204)

    def test_update_policy_association_for_region_and_service(self):
        self.check_service_client_function(
            self.client.update_policy_association_for_region_and_service,
            'tempest.lib.common.rest_client.RestClient.put',
            {},
            policy_id=self.FAKE_POLICY_ID,
            service_id=self.FAKE_SERVICE_ID,
            region_id=self.FAKE_REGION_ID,
            status=204)

    def test_show_policy_association_for_region_and_service(self):
        self.check_service_client_function(
            self.client.show_policy_association_for_region_and_service,
            'tempest.lib.common.rest_client.RestClient.get',
            {},
            policy_id=self.FAKE_POLICY_ID,
            service_id=self.FAKE_SERVICE_ID,
            region_id=self.FAKE_REGION_ID,
            status=204)

    def test_delete_policy_association_for_region_and_service(self):
        self.check_service_client_function(
            self.client.delete_policy_association_for_region_and_service,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            policy_id=self.FAKE_POLICY_ID,
            service_id=self.FAKE_SERVICE_ID,
            region_id=self.FAKE_REGION_ID,
            status=204)

    def _test_list_endpoints_for_policy(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_endpoints_for_policy,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_ENDPOINT_INFO,
            bytes_body,
            policy_id=self.FAKE_POLICY_ID,
            status=200)

    def test_list_endpoints_for_policy_with_str_body(self):
        self._test_list_endpoints_for_policy()

    def test_list_endpoints_for_policy_with_bytes_body(self):
        self._test_list_endpoints_for_policy(bytes_body=True)

    def _test_list_policy_for_endpoint(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_policy_for_endpoint,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_POLICY_INFO,
            bytes_body,
            endpoint_id=self.FAKE_ENDPOINT_ID,
            status=200)

    def test_list_policy_for_endpoint_with_str_body(self):
        self._test_list_policy_for_endpoint()

    def test_list_policy_for_endpoint_with_bytes_body(self):
        self._test_list_policy_for_endpoint(bytes_body=True)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.api.compute import base
from tempest.lib import decorators


class TestVersions(base.BaseV2ComputeTest):

    @decorators.idempotent_id('6c0a0990-43b6-4529-9b61-5fd8daf7c55c')
    @decorators.attr(type='smoke')
    def test_list_api_versions(self):
        """Test that a get of the unversioned url returns the choices doc.

        A key feature in OpenStack services is the idea that you can
        GET / on the service and get a list of the versioned endpoints
        that you can access. This comes back as a status 300
        request. It's important that this is available to API
        consumers to discover the API they can use.

        """
        result = self.versions_client.list_versions()
        versions = result['versions']
        # NOTE(sdague): at a later point we may want to loosen this
        # up, but for now this should be true of all Novas deployed.
        self.assertEqual(versions[0]['id'], 'v2.0',
                         "The first listed version should be v2.0")

    @decorators.idempotent_id('b953a29e-929c-4a8e-81be-ec3a7e03cb76')
    @decorators.attr(type='smoke')
    def test_get_version_details(self):
        """Test individual version endpoints info works.

        In addition to the GET / version request, there is also a
        version info document stored at the top of the versioned
        endpoints. This provides access to details about that
        endpoint, including min / max version if that implements
        microversions.

        This test starts with the version list, iterates all the
        returned endpoints, and fetches them. This will also ensure
        that all the version links are followable constructs which
        will help detect configuration issues when SSL termination
        isn't done completely for a site.

        """
        result = self.versions_client.list_versions()
        versions = result['versions']

        # iterate through all the versions that are returned and
        # attempt to get their version documents.
        for version in versions:
            links = [x for x in version['links'] if x['rel'] == 'self']
            self.assertEqual(
                len(links), 1,
                "There should only be 1 self link in %s" % version)
            link = links[0]
            # this is schema validated
            result = self.versions_client.get_version_by_url(link['href'])
            # ensure the new self link matches the old one
            newlinks = [x for x in result['version']['links']
                        if x['rel'] == 'self']
            self.assertEqual(links, newlinks)



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io
import random

from tempest.api.compute import base
from tempest.common import image as common_image
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class FlavorsV2NegativeTest(base.BaseV2ComputeTest):

    @decorators.attr(type=['negative'])
    @utils.services('image')
    @decorators.idempotent_id('90f0d93a-91c1-450c-91e6-07d18172cefe')
    def test_boot_with_low_ram(self):
        """Try boot a vm with lower than min ram

        Create an image with min_ram value
        Try to create server with flavor of insufficient ram size from
        that image
        """
        flavor = self.flavors_client.show_flavor(
            CONF.compute.flavor_ref)['flavor']
        min_img_ram = flavor['ram'] + 1
        size = random.randint(1024, 4096)
        image_file = io.BytesIO(data_utils.random_bytes(size))
        params = {
            'name': data_utils.rand_name('image'),
            'container_format': CONF.image.container_formats[0],
            'disk_format': CONF.image.disk_formats[0],
            'min_ram': min_img_ram
        }

        if CONF.image_feature_enabled.api_v1:
            params.update({'is_public': False})
            params = {'headers': common_image.image_meta_to_headers(**params)}
        else:
            params.update({'visibility': 'private'})

        image = self.images_client.create_image(**params)
        image = image['image'] if 'image' in image else image
        self.addCleanup(self.images_client.delete_image, image['id'])

        if CONF.image_feature_enabled.api_v1:
            self.images_client.update_image(image['id'], data=image_file)
        else:
            self.images_client.store_image_file(image['id'], data=image_file)

        self.assertEqual(min_img_ram, image['min_ram'])

        # Try to create server with flavor of insufficient ram size
        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server,
                          image_id=image['id'],
                          flavor=flavor['id'])



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators


class RolesTestJSON(base.BaseIdentityV2AdminTest):

    @classmethod
    def resource_setup(cls):
        super(RolesTestJSON, cls).resource_setup()
        cls.roles = list()
        for _ in range(5):
            role_name = data_utils.rand_name(name='role')
            role = cls.roles_client.create_role(name=role_name)['role']
            cls.addClassResourceCleanup(
                test_utils.call_and_ignore_notfound_exc,
                cls.roles_client.delete_role, role['id'])
            cls.roles.append(role)

    def _get_role_params(self):
        user = self.setup_test_user()
        tenant = self.tenants_client.show_tenant(user['tenantId'])['tenant']
        role = self.setup_test_role()
        return (user, tenant, role)

    def assert_role_in_role_list(self, role, roles):
        found = False
        for user_role in roles:
            if user_role['id'] == role['id']:
                found = True
        self.assertTrue(found, "assigned role was not in list")

    @decorators.idempotent_id('75d9593f-50b7-4fcf-bd64-e3fb4a278e23')
    def test_list_roles(self):
        """Return a list of all roles."""
        body = self.roles_client.list_roles()['roles']
        found = [role for role in body if role in self.roles]
        self.assertNotEmpty(found)
        self.assertEqual(len(found), len(self.roles))

    @decorators.idempotent_id('c62d909d-6c21-48c0-ae40-0a0760e6db5e')
    def test_role_create_delete(self):
        """Role should be created, verified, and deleted."""
        role_name = data_utils.rand_name(name='role-test')
        body = self.roles_client.create_role(name=role_name)['role']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.roles_client.delete_role, body['id'])
        self.assertEqual(role_name, body['name'])

        body = self.roles_client.list_roles()['roles']
        found = [role for role in body if role['name'] == role_name]
        self.assertNotEmpty(found)

        body = self.roles_client.delete_role(found[0]['id'])

        body = self.roles_client.list_roles()['roles']
        found = [role for role in body if role['name'] == role_name]
        self.assertEmpty(found)

    @decorators.idempotent_id('db6870bd-a6ed-43be-a9b1-2f10a5c9994f')
    def test_get_role_by_id(self):
        """Get a role by its id."""
        role = self.setup_test_role()
        role_id = role['id']
        role_name = role['name']
        body = self.roles_client.show_role(role_id)['role']
        self.assertEqual(role_id, body['id'])
        self.assertEqual(role_name, body['name'])

    @decorators.idempotent_id('0146f675-ffbd-4208-b3a4-60eb628dbc5e')
    def test_assign_user_role(self):
        """Assign a role to a user on a tenant."""
        (user, tenant, role) = self._get_role_params()
        self.roles_client.create_user_role_on_project(tenant['id'],
                                                      user['id'],
                                                      role['id'])
        roles = self.roles_client.list_user_roles_on_project(
            tenant['id'], user['id'])['roles']
        self.assert_role_in_role_list(role, roles)

    @decorators.idempotent_id('f0b9292c-d3ba-4082-aa6c-440489beef69')
    def test_remove_user_role(self):
        """Remove a role assigned to a user on a tenant."""
        (user, tenant, role) = self._get_role_params()
        user_role = self.roles_client.create_user_role_on_project(
            tenant['id'], user['id'], role['id'])['role']
        self.roles_client.delete_role_from_user_on_project(tenant['id'],
                                                           user['id'],
                                                           user_role['id'])

    @decorators.idempotent_id('262e1e3e-ed71-4edd-a0e5-d64e83d66d05')
    def test_list_user_roles(self):
        """List roles assigned to a user on tenant."""
        (user, tenant, role) = self._get_role_params()
        self.roles_client.create_user_role_on_project(tenant['id'],
                                                      user['id'],
                                                      role['id'])
        roles = self.roles_client.list_user_roles_on_project(
            tenant['id'], user['id'])['roles']
        self.assert_role_in_role_list(role, roles)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures

from tempest.lib import exceptions as lib_exc
from tempest.lib.services.compute import snapshots_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestSnapshotsClient(base.BaseServiceTest):

    FAKE_SNAPSHOT = {
        "createdAt": "2015-10-02T16:27:54.724209",
        "displayDescription": u"Another \u1234.",
        "displayName": u"v\u1234-001",
        "id": "100",
        "size": 100,
        "status": "available",
        "volumeId": "12"
    }

    FAKE_SNAPSHOTS = {"snapshots": [FAKE_SNAPSHOT]}

    def setUp(self):
        super(TestSnapshotsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = snapshots_client.SnapshotsClient(
            fake_auth, 'compute', 'regionOne')

    def _test_create_snapshot(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_snapshot,
            'tempest.lib.common.rest_client.RestClient.post',
            {"snapshot": self.FAKE_SNAPSHOT},
            to_utf=bytes_body, status=200,
            volume_id=self.FAKE_SNAPSHOT["volumeId"])

    def test_create_snapshot_with_str_body(self):
        self._test_create_snapshot()

    def test_create_shapshot_with_bytes_body(self):
        self._test_create_snapshot(bytes_body=True)

    def _test_show_snapshot(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_snapshot,
            'tempest.lib.common.rest_client.RestClient.get',
            {"snapshot": self.FAKE_SNAPSHOT},
            to_utf=bytes_body, snapshot_id=self.FAKE_SNAPSHOT["id"])

    def test_show_snapshot_with_str_body(self):
        self._test_show_snapshot()

    def test_show_snapshot_with_bytes_body(self):
        self._test_show_snapshot(bytes_body=True)

    def _test_list_snapshots(self, bytes_body=False, **params):
        self.check_service_client_function(
            self.client.list_snapshots,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SNAPSHOTS, to_utf=bytes_body, **params)

    def test_list_snapshots_with_str_body(self):
        self._test_list_snapshots()

    def test_list_snapshots_with_byte_body(self):
        self._test_list_snapshots(bytes_body=True)

    def test_list_snapshots_with_params(self):
        self._test_list_snapshots('fake')

    def test_delete_snapshot(self):
        self.check_service_client_function(
            self.client.delete_snapshot,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, status=202, snapshot_id=self.FAKE_SNAPSHOT['id'])

    def test_is_resource_deleted_true(self):
        module = ('tempest.lib.services.compute.snapshots_client.'
                  'SnapshotsClient.show_snapshot')
        self.useFixture(fixtures.MockPatch(
            module, side_effect=lib_exc.NotFound))
        self.assertTrue(self.client.is_resource_deleted('fake-id'))

    def test_is_resource_deleted_false(self):
        module = ('tempest.lib.services.compute.snapshots_client.'
                  'SnapshotsClient.show_snapshot')
        self.useFixture(fixtures.MockPatch(
            module, return_value={}))
        self.assertFalse(self.client.is_resource_deleted('fake-id'))



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.network import base
from tempest.common import utils
from tempest.common.utils import net_utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

CONF = config.CONF


class FloatingIPTestJSON(base.BaseNetworkTest):
    """Tests the following operations in the Neutron API:

        Create a Floating IP
        Update a Floating IP
        Delete a Floating IP
        List all Floating IPs
        Show Floating IP details
        Associate a Floating IP with a port and then delete that port
        Associate a Floating IP with a port and then with a port on another
        router

    v2.0 of the Neutron API is assumed. It is also assumed that the following
    options are defined in the [network] section of etc/tempest.conf:

        public_network_id which is the id for the external network present
    """

    @classmethod
    def skip_checks(cls):
        super(FloatingIPTestJSON, cls).skip_checks()
        if not utils.is_extension_enabled('router', 'network'):
            msg = "router extension not enabled."
            raise cls.skipException(msg)
        if not CONF.network.public_network_id:
            msg = "The public_network_id option must be specified."
            raise cls.skipException(msg)
        if not CONF.network_feature_enabled.floating_ips:
            raise cls.skipException("Floating ips are not available")

    @classmethod
    def resource_setup(cls):
        super(FloatingIPTestJSON, cls).resource_setup()
        cls.ext_net_id = CONF.network.public_network_id

        # Create network, subnet, router and add interface
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network, enable_dhcp=False)
        cls.router = cls.create_router(external_network_id=cls.ext_net_id)
        cls.create_router_interface(cls.router['id'], cls.subnet['id'])
        # Create two ports one each for Creation and Updating of floatingIP
        cls.ports = []
        for _ in range(2):
            port = cls.create_port(cls.network)
            cls.ports.append(port)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('62595970-ab1c-4b7f-8fcc-fddfe55e8718')
    def test_create_list_show_update_delete_floating_ip(self):
        """Test create/list/show/update/delete floating ip"""
        # Creates a floating IP
        body = self.floating_ips_client.create_floatingip(
            floating_network_id=self.ext_net_id,
            port_id=self.ports[0]['id'])
        created_floating_ip = body['floatingip']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.floating_ips_client.delete_floatingip,
            created_floating_ip['id'])
        self.assertIsNotNone(created_floating_ip['id'])
        self.assertIsNotNone(created_floating_ip['project_id'])
        self.assertIsNotNone(created_floating_ip['floating_ip_address'])
        self.assertEqual(created_floating_ip['port_id'], self.ports[0]['id'])
        self.assertEqual(created_floating_ip['floating_network_id'],
                         self.ext_net_id)
        self.assertIn(created_floating_ip['fixed_ip_address'],
                      [ip['ip_address'] for ip in self.ports[0]['fixed_ips']])
        # Verifies the details of a floating_ip
        floating_ip = self.floating_ips_client.show_floatingip(
            created_floating_ip['id'])
        shown_floating_ip = floating_ip['floatingip']
        self.assertEqual(shown_floating_ip['id'], created_floating_ip['id'])
        self.assertEqual(shown_floating_ip['floating_network_id'],
                         self.ext_net_id)
        self.assertEqual(shown_floating_ip['project_id'],
                         created_floating_ip['project_id'])
        self.assertEqual(shown_floating_ip['floating_ip_address'],
                         created_floating_ip['floating_ip_address'])
        self.assertEqual(shown_floating_ip['port_id'], self.ports[0]['id'])

        # Verify the floating ip exists in the list of all floating_ips
        floating_ips = self.floating_ips_client.list_floatingips()
        floatingip_id_list = list()
        for f in floating_ips['floatingips']:
            floatingip_id_list.append(f['id'])
        self.assertIn(created_floating_ip['id'], floatingip_id_list)
        # Associate floating IP to the other port
        floating_ip = self.floating_ips_client.update_floatingip(
            created_floating_ip['id'],
            port_id=self.ports[1]['id'])
        updated_floating_ip = floating_ip['floatingip']
        self.assertEqual(updated_floating_ip['port_id'], self.ports[1]['id'])
        self.assertEqual(updated_floating_ip['fixed_ip_address'],
                         self.ports[1]['fixed_ips'][0]['ip_address'])
        self.assertEqual(updated_floating_ip['router_id'], self.router['id'])

        # Disassociate floating IP from the port
        floating_ip = self.floating_ips_client.update_floatingip(
            created_floating_ip['id'],
            port_id=None)
        updated_floating_ip = floating_ip['floatingip']
        self.assertIsNone(updated_floating_ip['port_id'])
        self.assertIsNone(updated_floating_ip['fixed_ip_address'])
        self.assertIsNone(updated_floating_ip['router_id'])

        # Explicity test deletion of floating IP
        self.floating_ips_client.delete_floatingip(created_floating_ip['id'])

    @decorators.idempotent_id('e1f6bffd-442f-4668-b30e-df13f2705e77')
    def test_floating_ip_delete_port(self):
        """Test deleting floating ip's port

        1. Create a floating ip
        2. Create a port
        3. Update the floating ip's port_id to the created port
        4. Delete the port
        5. Verify that the port details are cleared from the floating ip
        """
        # Create a floating IP
        body = self.floating_ips_client.create_floatingip(
            floating_network_id=self.ext_net_id)
        created_floating_ip = body['floatingip']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.floating_ips_client.delete_floatingip,
            created_floating_ip['id'])
        # Create a port
        port = self.ports_client.create_port(
            network_id=self.network['id'],
            name=data_utils.rand_name(self.__class__.__name__))
        created_port = port['port']
        floating_ip = self.floating_ips_client.update_floatingip(
            created_floating_ip['id'],
            port_id=created_port['id'])
        # Delete port
        self.ports_client.delete_port(created_port['id'])
        # Verifies the details of the floating_ip
        floating_ip = self.floating_ips_client.show_floatingip(
            created_floating_ip['id'])
        shown_floating_ip = floating_ip['floatingip']
        # Confirm the fields are back to None
        self.assertEqual(shown_floating_ip['id'], created_floating_ip['id'])
        self.assertIsNone(shown_floating_ip['port_id'])
        self.assertIsNone(shown_floating_ip['fixed_ip_address'])
        self.assertIsNone(shown_floating_ip['router_id'])

    @decorators.idempotent_id('1bb2f731-fe5a-4b8c-8409-799ade1bed4d')
    def test_floating_ip_update_different_router(self):
        """Test associating a floating ip to a port on different router"""
        # Associate a floating IP to a port on a router
        body = self.floating_ips_client.create_floatingip(
            floating_network_id=self.ext_net_id,
            port_id=self.ports[1]['id'])
        created_floating_ip = body['floatingip']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.floating_ips_client.delete_floatingip,
            created_floating_ip['id'])
        self.assertEqual(created_floating_ip['router_id'], self.router['id'])
        network_name = data_utils.rand_name(self.__class__.__name__)
        network2 = self.networks_client.create_network(
            name=network_name)['network']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.networks_client.delete_network,
            network2['id'])
        subnet2 = self.create_subnet(network2)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.subnets_client.delete_subnet, subnet2['id'])
        router2 = self.create_router(external_network_id=self.ext_net_id)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.routers_client.delete_router, router2['id'])
        self.create_router_interface(router2['id'], subnet2['id'])
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.routers_client.remove_router_interface,
            router2['id'], subnet_id=subnet2['id'])
        port_other_router = self.create_port(network2)
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.ports_client.delete_port,
            port_other_router['id'])
        # Associate floating IP to the other port on another router
        floating_ip = self.floating_ips_client.update_floatingip(
            created_floating_ip['id'],
            port_id=port_other_router['id'])
        updated_floating_ip = floating_ip['floatingip']
        self.assertEqual(updated_floating_ip['router_id'], router2['id'])
        self.assertEqual(updated_floating_ip['port_id'],
                         port_other_router['id'])
        self.assertIsNotNone(updated_floating_ip['fixed_ip_address'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('36de4bd0-f09c-43e3-a8e1-1decc1ffd3a5')
    def test_create_floating_ip_specifying_a_fixed_ip_address(self):
        """Test creating floating ip with specified fixed ip"""
        body = self.floating_ips_client.create_floatingip(
            floating_network_id=self.ext_net_id,
            port_id=self.ports[1]['id'],
            fixed_ip_address=self.ports[1]['fixed_ips'][0]['ip_address'])
        created_floating_ip = body['floatingip']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.floating_ips_client.delete_floatingip,
            created_floating_ip['id'])
        self.assertIsNotNone(created_floating_ip['id'])
        self.assertEqual(created_floating_ip['fixed_ip_address'],
                         self.ports[1]['fixed_ips'][0]['ip_address'])
        floating_ip = self.floating_ips_client.update_floatingip(
            created_floating_ip['id'],
            port_id=None)
        self.assertIsNone(floating_ip['floatingip']['port_id'])

    @decorators.idempotent_id('45c4c683-ea97-41ef-9c51-5e9802f2f3d7')
    def test_create_update_floatingip_with_port_multiple_ip_address(self):
        """Test updating floating ip's fixed_ips to another ip of same port

        First we create a port with 2 fixed ips, then we create a floating ip
        with one of the fixed ips, and then we update the floating ip to
        another fixed ip of that port.
        """
        # Find out ips that can be used for tests
        list_ips = net_utils.get_unused_ip_addresses(
            self.ports_client,
            self.subnets_client,
            self.subnet['network_id'],
            self.subnet['id'],
            2)
        fixed_ips = [{'ip_address': list_ips[0]}, {'ip_address': list_ips[1]}]
        # Create port
        body = self.ports_client.create_port(
            network_id=self.network['id'],
            name=data_utils.rand_name(self.__class__.__name__),
            fixed_ips=fixed_ips)
        port = body['port']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ports_client.delete_port, port['id'])
        # Create floating ip
        body = self.floating_ips_client.create_floatingip(
            floating_network_id=self.ext_net_id,
            port_id=port['id'],
            fixed_ip_address=list_ips[0])
        floating_ip = body['floatingip']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.floating_ips_client.delete_floatingip,
                        floating_ip['id'])
        self.assertIsNotNone(floating_ip['id'])
        self.assertEqual(floating_ip['fixed_ip_address'], list_ips[0])
        # Update floating ip
        body = self.floating_ips_client.update_floatingip(
            floating_ip['id'], port_id=port['id'],
            fixed_ip_address=list_ips[1])
        update_floating_ip = body['floatingip']
        self.assertEqual(update_floating_ip['fixed_ip_address'],
                         list_ips[1])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.image import base
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class ImagesMemberNegativeTest(base.BaseV2MemberImageTest):
    """Negative tests of image members"""

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('b79efb37-820d-4cf0-b54c-308b00cf842c')
    def test_image_share_invalid_status(self):
        """Test updating image member status to invalid status should fail"""
        image_id = self._create_image()
        member = self.image_member_client.create_image_member(
            image_id, member=self.alt_tenant_id)
        self.assertEqual(member['status'], 'pending')
        self.assertRaises(lib_exc.BadRequest,
                          self.alt_image_member_client.update_image_member,
                          image_id, self.alt_tenant_id,
                          status='notavalidstatus')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('27002f74-109e-4a37-acd0-f91cd4597967')
    def test_image_share_owner_cannot_accept(self):
        """Test that image owner can't accept image shared to other member"""
        image_id = self._create_image()
        member = self.image_member_client.create_image_member(
            image_id, member=self.alt_tenant_id)
        self.assertEqual(member['status'], 'pending')
        self.assertNotIn(image_id, self._list_image_ids_as_alt())
        self.assertRaises(lib_exc.Forbidden,
                          self.image_member_client.update_image_member,
                          image_id, self.alt_tenant_id, status='accepted')
        self.assertNotIn(image_id, self._list_image_ids_as_alt())



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import six

from tempest.api.network import base
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators


class AllowedAddressPairTestJSON(base.BaseNetworkTest):
    """Tests the Neutron Allowed Address Pair API extension

    The following API operations are tested with this extension:

        create port
        list ports
        update port
        show port

    v2.0 of the Neutron API is assumed. It is also assumed that the following
    options are defined in the [network-feature-enabled] section of
    etc/tempest.conf

        api_extensions
    """

    @classmethod
    def skip_checks(cls):
        super(AllowedAddressPairTestJSON, cls).skip_checks()
        if not utils.is_extension_enabled('allowed-address-pairs', 'network'):
            msg = "Allowed Address Pairs extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(AllowedAddressPairTestJSON, cls).resource_setup()
        cls.network = cls.create_network()
        cls.create_subnet(cls.network)
        port = cls.create_port(cls.network)
        cls.ip_address = port['fixed_ips'][0]['ip_address']
        cls.mac_address = port['mac_address']

    @decorators.idempotent_id('86c3529b-1231-40de-803c-00e40882f043')
    def test_create_list_port_with_address_pair(self):
        """Create and list port with allowed address pair attribute"""
        allowed_address_pairs = [{'ip_address': self.ip_address,
                                  'mac_address': self.mac_address}]
        body = self.ports_client.create_port(
            network_id=self.network['id'],
            name=data_utils.rand_name(self.__class__.__name__),
            allowed_address_pairs=allowed_address_pairs)
        port_id = body['port']['id']
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        port_id)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ports_client.delete_port, port_id)

        # Confirm port was created with allowed address pair attribute
        body = self.ports_client.list_ports()
        ports = body['ports']
        port = [p for p in ports if p['id'] == port_id]
        msg = 'Created port not found in list of ports returned by Neutron'
        self.assertTrue(port, msg)
        self._confirm_allowed_address_pair(port[0], self.ip_address)

    def _update_port_with_address(self, address, mac_address=None, **kwargs):
        # Create a port without allowed address pair
        body = self.ports_client.create_port(
            network_id=self.network['id'],
            name=data_utils.rand_name(self.__class__.__name__))
        port_id = body['port']['id']
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        port_id)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ports_client.delete_port, port_id)
        if mac_address is None:
            mac_address = self.mac_address

        # Update allowed address pair attribute of port
        allowed_address_pairs = [{'ip_address': address,
                                  'mac_address': mac_address}]
        if kwargs:
            allowed_address_pairs.append(kwargs['allowed_address_pairs'])
        body = self.ports_client.update_port(
            port_id, allowed_address_pairs=allowed_address_pairs)
        allowed_address_pair = body['port']['allowed_address_pairs']
        six.assertCountEqual(self, allowed_address_pair,
                             allowed_address_pairs)

    @decorators.idempotent_id('9599b337-272c-47fd-b3cf-509414414ac4')
    def test_update_port_with_address_pair(self):
        """Update port with allowed address pair"""
        self._update_port_with_address(self.ip_address)

    @decorators.idempotent_id('4d6d178f-34f6-4bff-a01c-0a2f8fe909e4')
    def test_update_port_with_cidr_address_pair(self):
        """Update allowed address pair with cidr"""
        self._update_port_with_address(str(self.cidr))

    @decorators.idempotent_id('b3f20091-6cd5-472b-8487-3516137df933')
    def test_update_port_with_multiple_ip_mac_address_pair(self):
        """Update allowed address pair port with multiple ip and mac"""
        resp = self.ports_client.create_port(
            network_id=self.network['id'],
            name=data_utils.rand_name(self.__class__.__name__))
        newportid = resp['port']['id']
        self.addCleanup(self.ports_client.wait_for_resource_deletion,
                        newportid)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.ports_client.delete_port, newportid)
        ipaddress = resp['port']['fixed_ips'][0]['ip_address']
        macaddress = resp['port']['mac_address']

        # Update allowed address pair port with multiple ip and  mac
        allowed_address_pairs = {'ip_address': ipaddress,
                                 'mac_address': macaddress}
        self._update_port_with_address(
            self.ip_address, self.mac_address,
            allowed_address_pairs=allowed_address_pairs)

    def _confirm_allowed_address_pair(self, port, ip):
        msg = 'Port allowed address pairs should not be empty'
        self.assertTrue(port['allowed_address_pairs'], msg)
        ip_address = port['allowed_address_pairs'][0]['ip_address']
        mac_address = port['allowed_address_pairs'][0]['mac_address']
        self.assertEqual(ip_address, ip)
        self.assertEqual(mac_address, self.mac_address)


class AllowedAddressPairIpV6TestJSON(AllowedAddressPairTestJSON):
    _ip_version = 6



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class UsersNegativeTest(base.BaseIdentityV3AdminTest):
    """Negative tests of keystone users"""

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e75f006c-89cc-477b-874d-588e4eab4b17')
    def test_create_user_for_non_existent_domain(self):
        """Attempt to create a user in a non-existent domain should fail"""
        u_name = data_utils.rand_name('user')
        u_email = u_name + '@testmail.tm'
        u_password = data_utils.rand_password()
        self.assertRaises(lib_exc.NotFound, self.users_client.create_user,
                          name=u_name, password=u_password,
                          email=u_email,
                          domain_id=data_utils.rand_uuid_hex())

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('b3c9fccc-4134-46f5-b600-1da6fb0a3b1f')
    def test_authentication_for_disabled_user(self):
        """Attempt to authenticate for disabled user should fail"""
        password = data_utils.rand_password()
        user = self.setup_test_user(password)
        self.disable_user(user['name'], user['domain_id'])
        self.assertRaises(lib_exc.Unauthorized, self.token.auth,
                          username=user['name'],
                          password=password,
                          user_domain_id=CONF.identity.default_domain_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import base


class ExtensionsClient(base.BaseNetworkClient):

    def show_extension(self, ext_alias, **fields):
        """Show extension details.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#show-extension-details
        """
        uri = '/extensions/%s' % ext_alias
        return self.show_resource(uri, **fields)

    def list_extensions(self, **filters):
        """List extensions.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#list-extensions
        """
        uri = '/extensions'
        return self.list_resources(uri, **filters)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_54 import servers as servers254
# ****** Schemas changed in microversion 2.57 *****************

# Note(gmann): This is schema for microversion 2.57 which includes the
# 'user_data' in the Response body of the following APIs:
#    - ``POST '/servers/{server_id}/action (rebuild)``

user_data = {
    'oneOf': [
        {
            'type': 'string',
            'format': 'base64',
            'maxLength': 65535
        },
        {'type': 'null'}
    ]
}

rebuild_server = copy.deepcopy(servers254.rebuild_server)
rebuild_server['response_body']['properties']['server'][
    'properties'].update({'user_data': user_data})
rebuild_server['response_body']['properties']['server'][
    'required'].append('user_data')

rebuild_server_with_admin_pass = copy.deepcopy(
    servers254.rebuild_server_with_admin_pass)
rebuild_server_with_admin_pass['response_body']['properties']['server'][
    'properties'].update({'user_data': user_data})
rebuild_server_with_admin_pass['response_body']['properties']['server'][
    'required'].append('user_data')

# NOTE(gmann): Below are the unchanged schema in this microversion. We need
# to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged in microversion 2.57 since microversion 2.54 ***
get_server = copy.deepcopy(servers254.get_server)
list_servers_detail = copy.deepcopy(servers254.list_servers_detail)
update_server = copy.deepcopy(servers254.update_server)
list_servers = copy.deepcopy(servers254.list_servers)
show_server_diagnostics = copy.deepcopy(servers254.show_server_diagnostics)
get_remote_consoles = copy.deepcopy(servers254.get_remote_consoles)
list_tags = copy.deepcopy(servers254.list_tags)
update_all_tags = copy.deepcopy(servers254.update_all_tags)
delete_all_tags = copy.deepcopy(servers254.delete_all_tags)
check_tag_existence = copy.deepcopy(servers254.check_tag_existence)
update_tag = copy.deepcopy(servers254.update_tag)
delete_tag = copy.deepcopy(servers254.delete_tag)
attach_volume = copy.deepcopy(servers254.attach_volume)
show_volume_attachment = copy.deepcopy(servers254.show_volume_attachment)
list_volume_attachments = copy.deepcopy(servers254.list_volume_attachments)



# Copyright (c) 2019 Ericsson
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client
from tempest.lib.services.placement import base_placement_client


class PlacementClient(base_placement_client.BasePlacementClient):

    def list_allocation_candidates(self, **params):
        """List allocation candidates.

        For full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/placement/#list-allocation-candidates
        """
        url = '/allocation_candidates'
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_allocations(self, consumer_uuid):
        """List all allocation records for the consumer.

        For full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/placement/#list-allocations
        """
        url = '/allocations/%s' % consumer_uuid
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)
